//: /be/optiloading/cargo/CargoList.java
/*
 * This work is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2007 Tim Pintens. All rights reserved.
 */

package be.optiloading.cargo;

import java.awt.Component;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.ResourceBundle;

import javax.swing.JOptionPane;

import be.optiloading.settings.Settings;
import be.optiloading.ship.ShipData;

/**
 * CargoList class used to stow cargo objects in an ArrayList<Cargo>.
 * 
 * @author Tim Pintens, info@optiloading.be
 * @author <a target="_blank"
 *         href="http://www.optiloading.be">www.optiloading.be</a>
 */

@SuppressWarnings("serial")
public class CargoList extends ArrayList<Cargo>{
	private NumberFormat formatter;
	private ResourceBundle languageResource;
	float[] maxVolumes;
	
	/**
	 * Constructor.
	 */
	public CargoList(){
		languageResource = Settings.getInstance().getResourceBundle();
		formatter = NumberFormat.getInstance();
        formatter.setMaximumFractionDigits(3);
        formatter.setMinimumFractionDigits(3);
        maxVolumes = ShipData.getInstance().getCargoTankList().getMaxVolsArray();
	}
	
	/**
	 * Method to add a Cargo object to the list. Will check with the cargoTankList
	 * if the Cargo object can be added.
	 * 
	 * @param newCargo Cargo object you want to add.
	 * @param component Parent window for possible error message.
	 * @return Boolean value indicating if the cargo was added.
	 */
	public boolean add(Cargo newCargo, Component component){
		ShipData data = ShipData.getInstance();
		
		boolean returnvalue = true;
		
		if( (newCargo.getVolume() + getTotalVolume()) > data.getMaxcargovolume()){
			JOptionPane.showMessageDialog(component,
					String.format(languageResource.getString("messageMaxVolume"), 
							formatter.format(data.getMaxcargovolume() - getTotalVolume()),
							formatter.format(data.getMaxcargovolume())));
			returnvalue = false;
		}
		
		if((newCargo.getWeight()+ getTotalWeight()) > data.getMaxdeadweight()){
			JOptionPane.showMessageDialog(
					component,String.format(languageResource.getString("messageMaxWeight"), 
							formatter.format(data.getMaxdeadweight() - getTotalWeight()),
							formatter.format(data.getMaxdeadweight())));
			returnvalue = false;
		}
		
		if(returnvalue){
			if(getMinTanks(-1,newCargo.getVolume()) <= data.getNumberofcargotanks()){
				returnvalue = true;
			} else {
				JOptionPane.showMessageDialog(
						component,languageResource.getString("messageMaxTanks"));
				returnvalue = false;
			}
		}
		
		if(returnvalue){
			super.add(newCargo);
		}
		
		data = null;
		
		return returnvalue;
	}
	
	/**
	 * Get the total volume of all Cargo objects in the list.
	 * 
	 * @return Floating point value indicating the total volume of the cargo.
	 */
	public float getTotalVolume(){
		float returnvalue = 0f;
		for(Cargo x: this){
			returnvalue += x.getVolume();
		}
		return returnvalue;
	}
	
	/**
	 * Get the total weight of all Cargo objects in the list.
	 * 
	 * @return Floating point value indicating the total weight of the cargo.
	 */
	public float getTotalWeight(){
		float returnvalue = 0f;
		for(Cargo x: this){
			returnvalue += x.getWeight();
		}
		return returnvalue;
	}
	
	/**
	 * Check if the cargo id is not already in use.
	 * 
	 * @param newId String value containing the id that one whiches to add.
	 * @return Boolean value which is true if id is already in use.
	 */
	public boolean checkId(String newId){
		boolean returnvalue = false;
		for(int i = 0; i < this.size();i++){
			if(this.get(i).getCargoid().equals(newId)){
				returnvalue = true;
			}
		}
		return returnvalue;
	}
	
	/**
	 * Get the minimum number of tanks in use when 
	 * volume of cargo is replaced by vol.
	 * @param cargo Cargo that has to be edited.
	 * @param vol New volume
	 * @return Minimum number of tanks in use.
	 */
	public int getMinTanks(int cargo, float vol){
		int returnvalue = 0;
		int i = maxVolumes.length-1;
		while(vol > 0){
			vol -= maxVolumes[i];
			i--;
			returnvalue++;
		}
		for(int k = 0; k < size();k++){
			if(k == cargo){
				continue;
			}
			i = maxVolumes.length-1;
			vol = get(k).getVolume();
			while(vol > 0){
				vol -= maxVolumes[i];
				i--;
				returnvalue++;
			}
		}
		return returnvalue;
	}
	
}
