//: be/optiloading/cargo/CargoPanel.java
/*
 * This work is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2007 Tim Pintens. All rights reserved.
 */

package be.optiloading.cargo;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.MouseEvent;
import java.util.ResourceBundle;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultCellEditor;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.SwingConstants;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;

import be.optiloading.gui.MainFrame;
import be.optiloading.helpers.table.NumberEditor;
import be.optiloading.helpers.table.NumberRenderer;
import be.optiloading.settings.Settings;
import be.optiloading.ship.ShipData;

/**
 * CargoPanel used to display cargo table.
 * 
 * @author Tim Pintens, info@optiloading.be
 * @author <a target="_blank"
 *         href="http://www.optiloading.be">www.optiloading.be</a>
 */

@SuppressWarnings("serial")
public class CargoPanel extends JPanel{
	private CargoList cargoList;
	private JTable table;
	private JButton deleteButton;
	private JButton addButton;
	private JButton clearButton;
	private JScrollPane scroll;
	private String[] columnToolTips;
	private ResourceBundle languageResource;
	
	/**
	 * Constructor which builds the panel.
	 */
	public CargoPanel(){
		super(new BorderLayout());
		languageResource = Settings.getInstance().getResourceBundle();
		this.setBorder(new CompoundBorder(new TitledBorder(new EtchedBorder(),
				languageResource.getString("cargoinput")), new EmptyBorder(5, 5, 5, 5)));
		
		columnToolTips = new String[6];
		columnToolTips[0] = languageResource.getString("numberTooltip");
		columnToolTips[1] = languageResource.getString("cargoidTooltip");
		columnToolTips[2] = languageResource.getString("descriptionTooltip");
		columnToolTips[3] = languageResource.getString("volumeTooltip");
		columnToolTips[4] = languageResource.getString("weightTooltip");
		columnToolTips[5] = languageResource.getString("densityTooltip");
		
		cargoList = ShipData.getInstance().getCargoList();

		table = new JTable(new CargoModel(cargoList)){
		    
			
			public Component prepareRenderer(TableCellRenderer renderer, int row, int column){
		        Component returnComp = super.prepareRenderer(renderer, row, column);
		        Color alternateColor = new Color(236,243,254);
		        Color whiteColor = Color.WHITE;
		        if (!returnComp.getBackground().equals(getSelectionBackground())){
		            Color bg = (row % 2 == 0 ? alternateColor : whiteColor);
		            returnComp .setBackground(bg);
		            bg = null;
		        }

		        if(table.getSelectedRow() == row){
		        	returnComp.setForeground(Color.white);
		        }
		        
		        if((table.getSelectedRow() != row) &&
		        	(!((CargoModel) table.getModel()).isCellEditable(row, column))){
		        	returnComp.setForeground(Color.black);
		        }
		        
		        if((table.getSelectedRow() != row) &&
			        	(((CargoModel) table.getModel()).isCellEditable(row, column))){
			        	returnComp.setForeground(new Color(9,42,212));
			    }
		        
		        if((((CargoModel) table.getModel()).isCellEditable(row, column)) && 
		        		(table.getSelectedColumn() != column) && 
		        		(table.getSelectedRow() != row)){
		        	returnComp.setForeground(new Color(9,42,212));
		        }
		        
		        if (returnComp instanceof JComponent) {
		        	if((((CargoModel) table.getModel()).isCellEditable(row, column))){
		        		JComponent jc = (JComponent) returnComp;
		                jc.setToolTipText(Settings.getInstance().
		                		getResourceBundle().getString("clickEdit"));
			        }
	            }
		        
		        return returnComp;
			}
			
			public Component prepareEditor(TableCellEditor editor,int row, int column){
		    	Component returnComp = super.prepareEditor(editor, row, column);
		    	returnComp.setForeground(new Color(9,42,212));
		    	return returnComp;
		    }
		    
		    protected JTableHeader createDefaultTableHeader() {
		        return new JTableHeader(columnModel) {
		            public String getToolTipText(MouseEvent e) {
		                java.awt.Point p = e.getPoint();
		                int index = columnModel.getColumnIndexAtX(p.x);
		                int realIndex = 
		                        columnModel.getColumn(index).getModelIndex();
		                return columnToolTips[realIndex];
		            }
		        };
		    }
		};
		
		table.getTableHeader().setReorderingAllowed(false);
		((DefaultTableCellRenderer)table.getTableHeader().getDefaultRenderer())
		.setHorizontalAlignment(SwingConstants.CENTER);
		
		scroll = new JScrollPane(table,
				JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
				JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		scroll.addComponentListener(new ScrollResizeListener());
		
		TableColumn col = table.getColumnModel().getColumn(0);
		col.setMaxWidth(20);
		col.setMinWidth(20);
		
		col = table.getColumnModel().getColumn(1);
		col.setMaxWidth(35);
		col.setMinWidth(35);
		col.setCellEditor(new UpperCaseEditor());
		
		col = table.getColumnModel().getColumn(2);
		col.setMinWidth(150);
		for(int i = 3; i < table.getColumnCount();i++){
			col = table.getColumnModel().getColumn(i);
			col.setMinWidth(80);
		}
		
		table.setDefaultRenderer(Float.class, new NumberRenderer(2));
		table.setDefaultEditor(Float.class, new NumberEditor(2));
		table.setRowHeight(22);
		
		col = table.getColumnModel().getColumn(5);
		col.setCellEditor(new NumberEditor(3));
		col.setCellRenderer(new NumberRenderer(3));
		
		JTableHeader header = table.getTableHeader();
		Dimension dim = header.getPreferredSize();
		dim.height +=5;
		header.setPreferredSize(dim);
		
		JPanel buttonPanel = new JPanel();
		buttonPanel.setLayout(new BoxLayout(buttonPanel,BoxLayout.Y_AXIS));
		buttonPanel.setBorder(new EmptyBorder(0,6,0,0));
		addButton = new JButton(new ImageIcon(
				this.getClass().getResource("/images/add.png")));
		addButton.setToolTipText(languageResource.getString("addTooltip"));
		addButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
				AddFrame.getInstance().setVisible(true);
			}
		});
		
		deleteButton = new JButton(new ImageIcon(
				this.getClass().getResource("/images/delete.png")));
		deleteButton.setToolTipText(languageResource.getString("deleteTooltip"));
		deleteButton.addActionListener(new ActionListener(){

			public void actionPerformed(ActionEvent e) {
				if(table.getSelectedRowCount() != 0){
					int[] selected = table.getSelectedRows();
					for(int i = 0; i < table.getSelectedRowCount();i++){
						ShipData.getInstance().getCargoCompList().
							deleteComp(selected[i]-i);
						ShipData.getInstance().getTankCompList().
							deleteComp(selected[i]-i);
						cargoList.remove(selected[i]-i);
					}
					((CargoModel) table.getModel()).fireTableDataChanged();
					MainFrame.getInstance().updateInterface();
				}
			}
		});
		
		buttonPanel.add(addButton);
		buttonPanel.add(Box.createVerticalStrut(5));
		buttonPanel.add(deleteButton);
		
		JPanel clearPanel = new JPanel();
		clearPanel.setLayout(new BoxLayout(clearPanel,BoxLayout.Y_AXIS));
		clearPanel.setBorder(new EmptyBorder(0,6,0,0));
		clearButton = new JButton(new ImageIcon(
				this.getClass().getResource("/images/cross.png")));
		clearButton.setToolTipText(languageResource.getString("clearTooltip"));
		clearButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				ShipData.getInstance().getCargoCompList().clear();
				ShipData.getInstance().getTankCompList().clear();
				cargoList.clear();
				((CargoModel) table.getModel()).fireTableDataChanged();
				MainFrame.getInstance().updateInterface();
			}
		});
		clearPanel.add(clearButton);
		clearPanel.add(Box.createVerticalStrut(1));
		
		JPanel buttonmainpanel = new JPanel(new BorderLayout());
		buttonmainpanel.add(BorderLayout.NORTH,buttonPanel);
		buttonmainpanel.add(BorderLayout.SOUTH,clearPanel);
		
		this.add(BorderLayout.CENTER,scroll);
		this.add(BorderLayout.EAST,buttonmainpanel);
	}
	
	/**
	 * Lock cargo list.
	 */
	public void lock(){
		deleteButton.setEnabled(false);
		addButton.setEnabled(false);
		clearButton.setEnabled(false);
		((CargoModel) table.getModel()).setEditable(false);
	}
	
	/**
	 * Unlock cargo list.
	 */
	public void unlock(){
		deleteButton.setEnabled(true);
		addButton.setEnabled(true);
		clearButton.setEnabled(true);
		((CargoModel) table.getModel()).setEditable(true);
	}
	
	/**
	 * Update table
	 */
	public void update(){
		((CargoModel) table.getModel()).fireTableDataChanged();
	}
	
	//Resize listener for scrollPane
	private class ScrollResizeListener extends ComponentAdapter {
        public void componentResized(ComponentEvent e){
          if (table.getPreferredSize().width <=
          	scroll.getViewport().getExtentSize().width)
          {
            table.setAutoResizeMode(
              JTable.AUTO_RESIZE_ALL_COLUMNS);
          } else {
          	table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
          }
        }
      }
	
	private class UpperCaseEditor extends DefaultCellEditor {
		public UpperCaseEditor(){
			super(new UpperCaseField());
		}
	}
	
}
