//: /be/optiloading/compatibility/TankCompPanel.java
/*
 * This work is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2008 Tim Pintens. All rights reserved.
 */

package be.optiloading.compatibility;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.MouseEvent;
import java.util.ResourceBundle;

import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.SwingConstants;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableCellRenderer;

import be.optiloading.cargo.CargoList;
import be.optiloading.helpers.table.BooleanEditor;
import be.optiloading.helpers.table.BooleanRenderer;
import be.optiloading.settings.Settings;
import be.optiloading.ship.ShipData;
import be.optiloading.tank.TankList;

/**
 * CargoCompPanel used to display cargo-cargo compatibility table.
 * 
 * @author Tim Pintens, info@optiloading.be
 * @author <a target="_blank"
 *         href="http://www.optiloading.be">www.optiloading.be</a>
 */


@SuppressWarnings("serial")
public class TankCompPanel extends JPanel{
	private TankComp tankCompList;
	private CargoList cargoList;
	private TankList tankList;
	private JTable table;
	private JScrollPane scroll;
	private ResourceBundle languageResource;
	
	/**
	 * Constructor to make panel.
	 */
	public TankCompPanel(){
		super(new BorderLayout());
		languageResource = Settings.getInstance().getResourceBundle();
		this.setBorder(new CompoundBorder(new TitledBorder(new EtchedBorder(),
				languageResource.getString("tankcompinput")), new EmptyBorder(5, 5, 5, 5)));

		cargoList = ShipData.getInstance().getCargoList();
		tankList = ShipData.getInstance().getCargoTankList();
		tankCompList = ShipData.getInstance().getTankCompList();

		table = new JTable(new TankCompModel(tankCompList,cargoList,tankList)){
		    
			public Component prepareRenderer(TableCellRenderer renderer, int row, int column){
		        Component returnComp = super.prepareRenderer(renderer, row, column);
		        Color alternateColor = new Color(236,243,254);
		        Color whiteColor = Color.WHITE;
		        if (!returnComp.getBackground().equals(getSelectionBackground())){
		            Color bg = (row % 2 == 0 ? alternateColor : whiteColor);
		            returnComp.setBackground(bg);
		            bg = null;
		        }
		        
		        if (returnComp instanceof JComponent) {
		        	if((((TankCompModel) table.getModel()).isCellEditable(row, column)) &&
		        			(row != (column-1))){
		        		JComponent jc = (JComponent) returnComp;
		                jc.setToolTipText(Settings.getInstance().
		                		getResourceBundle().getString("clickChange"));
			        }
		        	
		        	if(((TankCompModel) table.getModel())
		        			.isCellEditable(row, column) == false){
		        		JComponent jc = (JComponent) returnComp;
		                jc.setToolTipText(ShipData.getInstance().getCargoList()
		                		.get(row).getDescription());
			        }
	            }
		        
		        return returnComp;
			}
			
			public void changeSelection(int rowIndex,
                    int columnIndex,
                    boolean toggle,
                    boolean extend){}
			
			protected JTableHeader createDefaultTableHeader() {
		        return new JTableHeader(columnModel) {
		            public String getToolTipText(MouseEvent e) {
		                java.awt.Point p = e.getPoint();
		                int index = columnModel.getColumnIndexAtX(p.x);
		                int realIndex = 
		                        columnModel.getColumn(index).getModelIndex();
		                return getTooltip(realIndex);
		            }
		        };
		    }
		};
		table.getTableHeader().setReorderingAllowed(false);
		((DefaultTableCellRenderer)table.getTableHeader().getDefaultRenderer())
		.setHorizontalAlignment(SwingConstants.CENTER);
		table.setRowHeight(22);
		table.setDefaultRenderer(Boolean.class, new BooleanRenderer(false));
		table.setDefaultEditor(Boolean.class, new BooleanEditor());
		
		scroll = new JScrollPane(table,
				JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
				JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		scroll.addComponentListener(new ScrollResizeListener());
		
		this.add(BorderLayout.CENTER,scroll);
	}
	
	//Resize listener for scrollPane
	private class ScrollResizeListener extends ComponentAdapter {
        public void componentResized(ComponentEvent e){
          if (table.getPreferredSize().width <=
          	scroll.getViewport().getExtentSize().width)
          {
            table.setAutoResizeMode(
              JTable.AUTO_RESIZE_ALL_COLUMNS);
          } else {
          	table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
          }
        }
     }
	
	/**
	 * Update TankCompPanel
	 */
	public void update(){
		((TankCompModel) table.getModel()).fireTableStructureChanged();
		((TankCompModel) table.getModel()).fireTableDataChanged();
	}
	
	// Get the description as tooltip for the column header
	private String getTooltip(int index){
		if(index == 0){
			return Settings.getInstance().getResourceBundle().getString("idTooltip");
		} else {
			return ShipData.getInstance().getCargoTankList().get(index-1).getTankid();
		}
		
	}
}