//: /be/optiloading/gui/AboutBox.java
/*
 * This work is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2007 Tim Pintens. All rights reserved.
 */

package be.optiloading.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.lang.reflect.Method;
import java.net.URL;
import java.util.Locale;
import java.util.ResourceBundle;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;

import be.optiloading.settings.Settings;

/**
 * AboutBox class which creates an about box containing author information.
 * 
 * @author Tim Pintens, info@optiloading.be
 * @author <a target="_blank"
 *         href="http://www.optiloading.be">www.optiloading.be</a>
 */

@SuppressWarnings("serial")
public class AboutBox extends JDialog {
	private ResourceBundle languageResource;
	private JLabel website;
	
	/**
	 * Constructor.
	 */
	public AboutBox() {
		super(MainFrame.getInstance());
		JPanel panel = new JPanel();
		panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));

		languageResource = Settings.getInstance().getResourceBundle();
		
		// Image
		URL imageURL = this.getClass()
				.getResource("/images/about.png");
		ImageIcon about = new ImageIcon(imageURL);
		JLabel image = new JLabel(about);
		image.setAlignmentX(Component.CENTER_ALIGNMENT);

		// Title
		JLabel title = new JLabel(languageResource.getString("programTitle"));
		title.setFont(new Font("Dialog", Font.BOLD, 16));
		title.setAlignmentX(Component.CENTER_ALIGNMENT);

		// Version
		JLabel version = new JLabel(languageResource.getString("version"));
		version.setFont(new Font("Dialog", Font.PLAIN, 10));
		version.setAlignmentX(Component.CENTER_ALIGNMENT);

		// License
		JEditorPane license = new JEditorPane();
		license.setEditable(false);
		JScrollPane scroll = new JScrollPane(license);
		scroll.setPreferredSize(new Dimension(285, 165));
		scroll.setAlignmentX(Component.CENTER_ALIGNMENT);
		
		String language = null;
		if(Settings.getInstance().getSystemLocale().equals(Locale.ENGLISH)){
			language = "/help/about_en.html";
		} else {
			language = "/help/about_nl.html";
		}
		
		try {
			license.setPage(AboutBox.class.getResource(language));
		} catch (Exception ex) {
			ex.printStackTrace();
			Log.getInstance().add(Log.JAVA,ex);
		}

		// Author
		CopyLabel author = new CopyLabel(languageResource.getString("author"));
		author.setFont(new Font("Dialog", Font.PLAIN, 12));
		author.setAlignmentX(Component.CENTER_ALIGNMENT);
		
		//Website
		website = new JLabel("http://www.optiloading.be");
		website.setFont(new Font("Dialog", Font.PLAIN, 12));
		website.setAlignmentX(Component.CENTER_ALIGNMENT);
		website.setForeground(new Color(0,120,227));
		website.addMouseListener(new MouseAdapter(){

			public void mouseClicked(MouseEvent e) {
				openURL("http://www.optiloading.be");
			}

			public void mouseEntered(MouseEvent e) {
				website.setForeground(new Color(0,69,131));
			}

			public void mouseExited(MouseEvent e) {
				website.setForeground(new Color(0,120,227));
			}
			
		});

		// Add to main panel
		panel.add(Box.createVerticalStrut(5));
		panel.add(image);
		panel.add(title);
		panel.add(version);
		panel.add(Box.createVerticalStrut(7));
		panel.add(scroll);
		panel.add(Box.createVerticalStrut(5));
		panel.add(author);
		panel.add(Box.createVerticalStrut(5));
		panel.add(website);
		panel.add(Box.createVerticalStrut(5));

		// Configure frame
		setTitle(languageResource.getString("title"));
		getContentPane().add(BorderLayout.CENTER, panel);
		setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		setResizable(false);
		pack();
		setLocationRelativeTo(null);
		setVisible(true);
	}
	
	// Open website link in browser.
	private void openURL(String url) {
		String osName = System.getProperty("os.name");
		try {
			if (osName.startsWith("Mac OS")) {
				Class<?> fileMgr = Class.forName("com.apple.eio.FileManager");
				Method openURL = fileMgr.getDeclaredMethod("openURL",
						new Class[] { String.class });
				openURL.invoke(null, new Object[] { url });
			} else if (osName.startsWith("Windows"))
				Runtime.getRuntime().exec(
						"rundll32 url.dll,FileProtocolHandler " + url);
			else { //assume Unix or Linux
				String[] browsers = { "firefox", "opera", "konqueror",
						"epiphany", "mozilla", "netscape","iceweasel",
						"seamonkey"};
				String browser = null;
				for (int count = 0; count < browsers.length && browser == null; count++)
					if (Runtime.getRuntime().exec(
							new String[] { "which", browsers[count] })
							.waitFor() == 0)
						browser = browsers[count];
				if (browser == null)
					throw new Exception("Could not find web browser");
				else
					Runtime.getRuntime().exec(new String[] { browser, url });
			}
		} catch (Exception e) {
			JOptionPane.showMessageDialog(this, 
					languageResource.getString("MsgWebsite") + ":\n"
					+ e.getLocalizedMessage());
		}
	}
	
	private class CopyLabel extends JTextField {
		public CopyLabel(String text) {
			super(text);
			JLabel label = new JLabel();
			setBorder(label.getBorder());
			setHorizontalAlignment(JTextField.CENTER);
			setBackground(label.getBackground());
		}

		public boolean isEditable() {
			return false;
		}

		public boolean isOpaque() {
			return false;
		}
	}

}