//: /be/optiloading/gui/Log.java
/*
 * This work is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2007 Tim Pintens. All rights reserved.
 */

package be.optiloading.gui;

import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintStream;
import java.sql.Time;
import java.util.ResourceBundle;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.border.BevelBorder;
import javax.swing.border.EmptyBorder;

import be.optiloading.helpers.files.FileHelper;
import be.optiloading.helpers.files.TxtFilter;
import be.optiloading.settings.Settings;

/**
 * Log class which holds all log information.
 * 
 * @author Tim Pintens, info@optiloading.be
 * @author <a target="_blank"
 *         href="http://www.optiloading.be">www.optiloading.be</a>
 */

@SuppressWarnings("serial")
public class Log extends JFrame{
	private JTextArea log;
	private Time time;
	private boolean printtime;
	private static Log uniqueInstance;
	private String[] categories;
	private int lastCategory;
	private ResourceBundle languageResource;
	private boolean firstLine = true;
	
	//Categories
	public static final int JAVA = 0;
	public static final int OPTILOADING = 1;
	public static final int LP = 2;
	public static final int SA = 3;
	
	/**
	 * Get singleton instance.
	 * 
	 * @return Log object.
	 */
	public static Log getInstance(){
		if (uniqueInstance == null) {
			uniqueInstance = new Log();
		}
		return uniqueInstance;
	}
	
	//Singleton constructor.
	private Log(){
		super();
		time = new Time(System.currentTimeMillis());
		languageResource = Settings.getInstance().getResourceBundle();
		categories = new String[4];
		categories[0] = "Java";
		categories[1] = languageResource.getString("programTitle");
		categories[2] = languageResource.getString("linearprogramming");
		categories[3] = languageResource.getString("simulatedannealing");
		lastCategory = -1;
		
		log = new JTextArea();
		log.setFont(new Font("monaco",Font.PLAIN,10));
		log.setEditable(false);
		log.setText("");
		
		JScrollPane scroll = new JScrollPane(log,
				JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
				JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		scroll.setBorder(new BevelBorder(BevelBorder.LOWERED));
		
		JPanel scrollPanel = new JPanel(new BorderLayout());
		scrollPanel.setBorder(new EmptyBorder(5,5,5,5));
		scrollPanel.add(BorderLayout.CENTER,scroll);
		
		JButton save = new JButton(languageResource.getString("saveButton"));
		save.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
				File file = FileHelper.getValidFile(".txt",new TxtFilter());
				if (file != null){
					try{
						FileWriter fwriter = new FileWriter(file);
						fwriter.write(log.getText());
						fwriter.close();
					} catch (Exception ex){
						JOptionPane.showMessageDialog(uniqueInstance,
								languageResource.getString("saveError"));
						uniqueInstance.add(JAVA,ex);
						ex.printStackTrace();
					}
				}
				uniqueInstance.requestFocus();
			}
		});
		JPanel buttonPanel = new JPanel(new BorderLayout());
		buttonPanel.setBorder(new EmptyBorder(5,5,10,10));
		buttonPanel.add(BorderLayout.EAST,save);
		
		getContentPane().add(BorderLayout.CENTER,scrollPanel);
		getContentPane().add(BorderLayout.SOUTH,buttonPanel);
		setTitle(languageResource.getString("logTitle"));
		setDefaultCloseOperation(JFrame.HIDE_ON_CLOSE);
		setIconImage(new ImageIcon(this.getClass().getResource(
		"/images/icon.png")).getImage());
		setSize(600,500);
		setResizable(true);
		setLocationRelativeTo(null);
	}
	
	/**
	 * Add information to log.
	 * @param category for which category.
	 * @param s String containing information.
	 */
	public void add(int category, String s){
		time.setTime(System.currentTimeMillis());
		String timeString = "[" + time.toString() + "] ";
		if(category != lastCategory){
			if(firstLine){
				log.append(timeString + categories[category].toUpperCase()+"\n");
				firstLine = false;
			} else {
				log.append("\n");
				log.append(timeString + categories[category].toUpperCase()+"\n");
			}
			
		} else {
			log.append("\n");
		}
		
		log.append(timeString + "\t" + s);
		log.setCaretPosition(log.getDocument().getLength());
		lastCategory = category;
	}
	
	/**
	 * Add information to log.
	 * @param category for which category.
	 * @param b char containing information.
	 */
	public void append(int category, char b){
		time.setTime(System.currentTimeMillis());
		String timeString = "[" + time.toString() + "] ";
		
		if(category != lastCategory){
			log.append(timeString + categories[category].toUpperCase()+"\n");
			printtime = true;
		}
		
		Character newLine = new Character('\n');
    	if(newLine.equals(b)){
    		log.append(String.valueOf(b));
    		printtime = true;
    	} else {
    		if(printtime){
    			log.append(timeString + "\t");
    			printtime = false;
    		}
    		log.append(String.valueOf(b));
    	}
    	log.setCaretPosition(log.getDocument().getLength());
		lastCategory = category;
	}
	
	/**
	 * Print all java messages and erros in log window.
	 *
	 */
	public void setStdout(){
		PrintStream out = new PrintStream(new TextAreaOutputStream());
		System.setOut(out);
		System.setErr(out);
	}
	
	/**
	 * Add exception to log.
	 * @param category for which category.
	 * @param e Exception which you want to add to the log.
	 */
	public void add(int category, Exception e){
		add(Log.JAVA, e.toString());
		for(int i = 0; i < e.getStackTrace().length;i++){
			add(Log.JAVA, e.getStackTrace()[i].toString());
		}
	}
	
	/**
	 * Overwritten method to make log visible.
	 * @param b Boolean which is true when log should be visible.
	 */
	public void setVisible(boolean b){
		super.setVisible(b);
		log.setCaretPosition(log.getDocument().getLength());
	}
	
	// Class which allows LP Solve output to log
	private class TextAreaOutputStream extends OutputStream {
		public void write( int b ) throws IOException {
	        Log.getInstance().append(Log.JAVA, (char) b);
	    }  
	}

}
