//: be/optiloading/help/HelpTree.java
/*
 * This work is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2007 Tim Pintens. All rights reserved.
 */

package be.optiloading.help;

import java.io.IOException;
import java.io.InputStream;
import java.util.ResourceBundle;

import javax.swing.ImageIcon;
import javax.swing.JOptionPane;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeSelectionModel;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import be.optiloading.gui.Log;
import be.optiloading.settings.Settings;

/**
 * HelpTree class reads the help content from a xml file and loads it into
 * a JTree.
 * 
 * @author Tim Pintens, info@optiloading.be
 * @author <a target="_blank"
 *         href="http://www.optiloading.be">www.optiloading.be</a>
 */

@SuppressWarnings("serial")
public class HelpTree extends JTree {
	private static final String helpxml = "/help/helpcontent.xml";
	private DefaultMutableTreeNode top;
	private ResourceBundle languageResource;
	
	/**
	 * Constructor of the HelpTree.
	 */
	public HelpTree(){
		super();
		top = null;
		languageResource = Settings.getInstance().getResourceBundle();
		((DefaultTreeModel) getModel()).setRoot(createNodes());
		getSelectionModel().setSelectionMode(
				TreeSelectionModel.SINGLE_TREE_SELECTION);
		DefaultTreeCellRenderer renderer = new DefaultTreeCellRenderer();
		renderer.setOpenIcon(new ImageIcon(this.getClass()
				.getResource("/images/opened.gif")));
		renderer.setClosedIcon(new ImageIcon(this.getClass()
				.getResource("/images/closed.gif")));
		renderer.setLeafIcon(new ImageIcon(this.getClass()
				.getResource("/images/leaf.gif")));
		setCellRenderer(renderer);
	}

	// Create the nodes
	private DefaultMutableTreeNode createNodes() {

		try {
			InputStream inputSource = this.getClass().
			getResource(helpxml).openStream();
			
			SAXParserFactory factory = SAXParserFactory.newInstance();
			factory.setValidating(true);
			factory.setNamespaceAware(false);
			
			SAXParser parser = factory.newSAXParser();
			parser.parse(inputSource, new TreeHandler());
		
		} catch (IOException e) {
			e.printStackTrace();
			Log.getInstance().add(Log.JAVA,e);
			JOptionPane.showMessageDialog(this, 
					languageResource.getString("exceptionMessage"),
					languageResource.getString("exceptionTitle"),
					JOptionPane.WARNING_MESSAGE);
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
			Log.getInstance().add(Log.JAVA,e);
			JOptionPane.showMessageDialog(this, 
					languageResource.getString("exceptionMessage"),
					languageResource.getString("exceptionTitle"),
					JOptionPane.WARNING_MESSAGE);
		} catch (SAXException e) {
			e.printStackTrace();
			Log.getInstance().add(Log.JAVA,e);
			JOptionPane.showMessageDialog(this, 
					languageResource.getString("exceptionMessage"),
					languageResource.getString("exceptionTitle"),
					JOptionPane.WARNING_MESSAGE);
		}

		return top;
	}
	
	// Responds to the callback events from the parser
	class TreeHandler extends DefaultHandler{
		private DefaultMutableTreeNode current;
		
		public void endElement(String arg0, String arg1, String arg2)
				throws SAXException {
			current = (DefaultMutableTreeNode) current.getParent();
		}

		public void startElement(String arg0, String arg1, String arg2,
				Attributes att) throws SAXException {
			if(top == null){
				String title = att.getValue("title");
				String file = att.getValue("file");
				top = new DefaultMutableTreeNode(
					  new HelpInfo(title,file));
				current = top;
			} else{
				String title = att.getValue("title");
				String file = att.getValue("file");
				DefaultMutableTreeNode element = new DefaultMutableTreeNode(
						new HelpInfo(title,file));
				current.add(element);
				current = element;
			}
		}
	}
}
