//: /be/optiloading/helpers/files/FileHelper.java
/*
 * This work is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2008 Tim Pintens. All rights reserved.
 */

package be.optiloading.helpers.files;

import java.io.File;
import java.io.InputStream;
import java.net.URL;
import java.util.prefs.Preferences;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileFilter;
import javax.xml.XMLConstants;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import javax.xml.validation.Validator;

import org.w3c.dom.Document;

import be.optiloading.gui.Log;
import be.optiloading.gui.MainFrame;
import be.optiloading.gui.MainMenu;
import be.optiloading.settings.Settings;

/**
 * FileHelper to perform commenly used File procedures.
 * 
 * @author Tim Pintens, info@optiloading.be
 * @author <a target="_blank"
 *         href="http://www.optiloading.be">www.optiloading.be</a>
 */

public class FileHelper {
	private static Preferences prefs = Preferences.userNodeForPackage(MainMenu.class);
	
	/**
	 * Open the file dialog and get a valid file. Also making sure not to overwrite
	 * files.
	 * @param extension String extension of the file.
	 * @param filter FileFilter to filter bad extensions.
	 * @return Valid File object.
	 */
	public static File getValidFile(String extension, FileFilter filter){
		JFileChooser fc = new JFileChooser();
		fc.setCurrentDirectory(new File(prefs.get("currentDir", 
				fc.getCurrentDirectory().getPath())));
		fc.setFileSelectionMode(JFileChooser.FILES_ONLY);
		fc.setMultiSelectionEnabled(false);
		fc.setFileFilter(filter);
		File returnvalue = null;
		while(true){
			int returnVal = fc.showSaveDialog(MainFrame.getInstance());
			if (returnVal == JFileChooser.APPROVE_OPTION){
				File file = fc.getSelectedFile();
				String filename = file.getName();
				if(!filename.endsWith(extension)){
					String newFilename = file.getPath() + extension;
					file = new File(newFilename);
				}
				if(file.exists()){
					if (JOptionPane.showConfirmDialog(null,
							Settings.getInstance().getResourceBundle()
								.getString("overwriteFile"), 
							Settings.getInstance().getResourceBundle()
								.getString("overwriteTitle"),
					        JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION){
						returnvalue = file;
						break;
					}
				} else {
					returnvalue = file;
					break;
				}
			} else {
				break;
			}
		}
		prefs.put("currentDir", fc.getCurrentDirectory().getPath());
		return returnvalue;
	}
	
	/**
	 * Check the xml file against the xsd schema.
	 * @param file The xml file.
	 * @param xsdURL The xsd URL.
	 * @return True if file is valid, otherwise false.
	 */
	public static boolean checkFile(File file, InputStream stream ,URL xsdURL){
		boolean returnvalue = true;
		try{
			SchemaFactory schemaFactory = SchemaFactory.newInstance(
					XMLConstants.W3C_XML_SCHEMA_NS_URI );
			Schema schemaXSD = schemaFactory.newSchema(xsdURL);
			Validator validator = schemaXSD.newValidator();
			DocumentBuilder parser = DocumentBuilderFactory.
				newInstance().newDocumentBuilder();
			Document document = null;
			if(file == null){
				document = parser.parse(stream);
			} else {
				document = parser.parse(file);
			}
			validator.validate( new DOMSource(document) );
			if(stream != null){
				stream.close();
			}
		} catch (Exception e){
			returnvalue = false;
			Log.getInstance().add(Log.JAVA, e);
		}
		return returnvalue;
	}
}
