//: be/optiloading/helpers/table/NumberRenderer.java
/*
 * This work is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2007 Tim Pintens. All rights reserved.
 */

package be.optiloading.helpers.table;

import java.text.NumberFormat;

import javax.swing.SwingConstants;
import javax.swing.table.DefaultTableCellRenderer;

/**
 * NumberRenderer to render numbers in table cells.
 * 
 * @author Tim Pintens, info@optiloading.be
 * @author <a target="_blank"
 *         href="http://www.optiloading.be">www.optiloading.be</a>
 */

@SuppressWarnings("serial")
public class NumberRenderer extends DefaultTableCellRenderer {
    NumberFormat formatter;
    int fractionDigits;
    
    /**
     * Constructor.
     */
    public NumberRenderer(int fractionDigits) { 
    	super();
    	this.fractionDigits = fractionDigits;
    }

    /**
     * Format value.
     */
    public void setValue(Object value) {
        if (formatter==null) {
            formatter = NumberFormat.getInstance();
            formatter.setMaximumFractionDigits(fractionDigits);
            formatter.setMinimumFractionDigits(fractionDigits);
            setHorizontalAlignment(SwingConstants.RIGHT);
        }
        String val = "";
        if(value != null){
        	val = formatter.format(value);
        }
        if(formatter.format(new Double(-0d)).equals(val)){
        	val = formatter.format(new Double(0d));
        }
        setText(val);
    }

}