//: /be/optiloading/optimization/Sa.java
/*
 * This work is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2008 Tim Pintens. All rights reserved.
 */


package be.optiloading.optimization;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.text.NumberFormat;
import java.util.Arrays;
import java.util.ResourceBundle;

import javax.swing.JOptionPane;

import be.optiloading.cargo.Cargo;
import be.optiloading.cargo.CargoList;
import be.optiloading.compatibility.CargoComp;
import be.optiloading.compatibility.SegregationComp;
import be.optiloading.compatibility.TankComp;
import be.optiloading.gui.Log;
import be.optiloading.gui.MainFrame;
import be.optiloading.gui.ProgressPanel;
import be.optiloading.settings.Settings;
import be.optiloading.ship.ShipData;
import be.optiloading.ship.StabilityData;
import be.optiloading.tank.TankList;

/**
 * Sa to perform simulated annealing algorithm.
 * 
 * @author Tim Pintens, info@optiloading.be
 * @author <a target="_blank"
 *         href="http://www.optiloading.be">www.optiloading.be</a>
 */
public class Sa extends Thread{
	private TankList cargoTanks;
	private CargoList cargoList;
	private TankComp tankCompList;
	private CargoComp cargoCompList;
	private SegregationComp segregationCompList;
	private ShipData data;
	private SectionList sectionList;
	private int i;
	private int j;
	private int z;
	private float displacement;
    private float lcb;
    private float mctc;
    private float lcf;
	private float totalCargoVolumes;
	private float totalTankVolumes;
	private ResourceBundle languageResource;
	private long begintime;
	private boolean debug;
	private float[] tankVolumes;
	private float[] cargoVolumes;
	private volatile boolean start = true;
	private int totalSwaps;
	private int totalNoSwap;
	private boolean step = false;
	
	// When random = true then all SA procedures will be more random.
	private boolean random = true;
	
	//Annealing settings
	private float tempReductionFactor;
	private int maxAnnealSteps;
	private int swapsPerStep;
	private int trailsPerStep;
	private float tempFactor;
	private boolean trim;
	private boolean heel;
	private boolean compatibility;
	private boolean strength;
	private boolean usedtanks;
	private float trimfactor;
	private float heelfactor;
	private float compatibilityfactor;
	private float strengthfactor;
	private float usedtanksfactor;
	private float desiredtrim;
	private float desiredheel;
	private int numberOfRestarts;
	
	//Test settings
	private static float percentfull = 0.75f;
	private static int numberofloads = 4;
	private static float ttempReductionFactor = 0.95f;
	private static int tmaxAnnealSteps = 1000;
	private static int tswapsPerStep = 100;
	private static int ttrailsPerStep = 400;
	private float ttempFactor = 100;
	private static boolean ttrim = true;
	private static boolean theel = true;
	private static boolean tcompatibility = true;
	private static boolean tstrength = true;
	private static boolean tusedtanks = true;
	private static float ttrimfactor = 1f;
	private static float theelfactor = 1f;
	private static float tcompatibilityfactor = 1f;
	private static float tstrengthfactor = 1f;
	private static float tusedtanksfactor = 1f;
	private static int numberofcomp = 1;
	private static float tdesiredtrim = 0f;
	private static float tdesiredheel = 0f;
	private static int tnumberOfRestarts = 5;
	private static boolean newloads = true; 
	private static float[] loads;

	// Best and current solution and energy level
	private float temperature;
	private Solution sol;
	private Solution bestSol;
	private float bestSolEnergy;
	private float solEnergy;
	
	/**
	 * Main test procedure
	 * @param args for main program. Input from command line.
	 */
	public static void main(String[] args){
		new Sa(true,false).run();
		System.exit(0);
	}
	
	/**
	 * Procedure to interrupt procedure and wait till return is pressed.
	 */
	@SuppressWarnings("unused")
	private static void pressRet() {
		System.out.print("\n[Press return to continue or type 'q' to quit] ");
		BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
		try {
			String line = br.readLine();
			if (line == null || "q".equals(line.trim())) {
				System.out.println("Test terminated.");
				System.exit(0);
			}
		} 
		catch (IOException e) {
			System.exit(0);
		}
		System.out.println("");
	}
	
	/**
	 * Constructor.
	 * @param debug  If true output will be written to command line.
	 * @param step If true SA algorithm can be performed in steps.
	 */
	public Sa(boolean debug, boolean step){
		// set begin time
		begintime = System.currentTimeMillis();
		languageResource = Settings.getInstance().getResourceBundle();
		this.debug = debug;
		this.step = step;
		// Load settings and variables 
		data = ShipData.getInstance();
		if(!debug){
			loadSettings();
		} else {
			loadTestProcedure();
		}
		cargoTanks = data.getCargoTankList();
		cargoList = data.getCargoList();
		tankCompList = data.getTankCompList();
		cargoCompList = data.getCargoCompList();
		segregationCompList = data.getSegregationCompList();
		cargoTanks.emptyTanks(true);
		i = cargoList.size();
		j = cargoTanks.size();
		float currdisplacement = data.getDisplacement();
	    displacement = cargoList.getTotalWeight() + 
	    data.getMiscTankList().getTotalWeight() + data.getLightship();
	    StabilityData stabData = data.getStabilityData();
	    stabData.setDisplacement(displacement);
	    stabData.setDontcheck(true);
	    lcb = stabData.getLcb();
	    mctc = stabData.getMctc();
	    lcf = stabData.getLcf();
	    stabData.setDontcheck(false);
	    stabData.setDisplacement(currdisplacement);
	    if(strength){
	    	sectionList = new SectionList(displacement, lcb, mctc, 
	    			lcf, desiredtrim);
		    z = sectionList.size();
	    } else {
	    	z = 0;
	    }
	    tankVolumes = new float[j];
		cargoVolumes = new float[i];
		totalCargoVolumes = 0f;
		totalTankVolumes = 0f;
		for(int k = 0; k < j; k++){
			tankVolumes[k] = cargoTanks.get(k).getMaxvolume();
			totalTankVolumes += tankVolumes[k];
		}
		for(int k = 0; k < i; k++){
			cargoVolumes[k] = cargoList.get(k).getVolume();
			totalCargoVolumes += cargoVolumes[k];
		}
		
		// Create initial solution for step algorithm
		if(step){
			sol = new Solution();
			bestSol = (Solution) sol.clone();
			bestSolEnergy = sol.getEnergy();
			temperature = (tempFactor*maxAnnealSteps*tempReductionFactor*
					((usedtanksfactor+trimfactor+
					heelfactor+compatibilityfactor+strengthfactor)/j));
		}
	}
	
	//Load test procedure
	private void loadTestProcedure(){
		data.initializeShipData("/data/ship.xml");
		data.initializeStabilityData("/data/hydrostatic.xml", "/data/kn.xml");
		data.initializeStrengthData("/data/lightship.xml", "/data/buoyancy.xml");
		data.initializeCargoTankList("/data/cargotanks.xml");
		data.initializeBallastTankList("/data/ballasttanks.xml");
		data.initializeMiscTankList("/data/misctanks.xml");
		data.initializeCargoList();
		data.initializeCargoCompList();
		data.initializeTankCompList();
		
		float totalvol = 2353.376f*percentfull;
		if(numberofloads == 1){
			data.getCargoList().add(new Cargo("L0"
					, "Load 0", totalvol,1),null);
			data.getCargoCompList().addComp();
			data.getTankCompList().addComp();
		} 
		else {
			if(newloads){
				loads = generateLoadList(totalvol,numberofloads);
			}
			for(int k = 0; k < numberofloads; k++){
				data.getCargoList().add(new Cargo("L"+String.valueOf(k)
						, "Load "+String.valueOf(k), loads[k],1),null);
				data.getCargoCompList().addComp();
				data.getTankCompList().addComp();
			}
		}
		
		i = data.getCargoList().size();
		j = data.getCargoTankList().size();
		
		// Generate random compatibility constraints
		int countCargocomp = 0;
		int maxCargocomp = (int) (((i*i)-i)/2);
		int countTankcomp = 0;
		int maxTankcomp = i*j;
		for(int k = 0; k < numberofcomp; k++){
			if(Math.random() > 0.5f){
				if(countCargocomp >= maxCargocomp){
					boolean value = true;
					int horz = 0;
					int vert = 0;
					while(value){
						horz = new Float(Math.random()*i).intValue();
						vert = new Float(Math.random()*j).intValue();
						value = data.getTankCompList().get(horz, vert);
					}
					data.getTankCompList().set(true, horz, vert);
					countTankcomp++;
				} else {
					boolean value = true;
					int horz = 0;
					int vert = 0;
					while(value){
						horz = new Float(Math.random()*i).intValue();
						vert = new Float(Math.random()*i).intValue();
						if(horz != vert){
							value = data.getCargoCompList().get(horz, vert);
						}
					}
					data.getCargoCompList().set(true, horz, vert);
					countCargocomp++;
				}
			} else {
				if(countTankcomp >= maxTankcomp){
					boolean value = true;
					int horz = 0;
					int vert = 0;
					while(value){
						horz = new Float(Math.random()*i).intValue();
						vert = new Float(Math.random()*i).intValue();
						if(horz != vert){
							value = data.getCargoCompList().get(horz, vert);
						}
					}
					data.getCargoCompList().set(true, horz, vert);
					countCargocomp++;
				} else {
					boolean value = true;
					int horz = 0;
					int vert = 0;
					while(value){
						horz = new Float(Math.random()*i).intValue();
						vert = new Float(Math.random()*j).intValue();
						value = data.getTankCompList().get(horz, vert);
					}
					data.getTankCompList().set(true, horz, vert);
					countTankcomp++;
				}	
			}
		}
		
		// Set test parameters
		tempReductionFactor = ttempReductionFactor;
		maxAnnealSteps = tmaxAnnealSteps;
		swapsPerStep = tswapsPerStep;
		trailsPerStep = ttrailsPerStep;
		tempFactor = ttempFactor;
		trim = 	ttrim;
		heel = theel;
		compatibility = tcompatibility;
		strength = tstrength;
		usedtanks = tusedtanks;
		trimfactor = ttrimfactor;
		heelfactor = theelfactor;
		compatibilityfactor = tcompatibilityfactor;
		strengthfactor = tstrengthfactor;
		usedtanksfactor = tusedtanksfactor;
		desiredtrim = tdesiredtrim;
		desiredheel = tdesiredheel;
		numberOfRestarts = tnumberOfRestarts;
	}
	
	// Generate random load list
	private float[] generateLoadList(float totalsum, int parts){
		float[] splitters = new float[parts-1];
		for (int k = 0;k<numberofloads-1;k++) {
		    splitters[k] = (float)(Math.random()*totalsum);
		}
		Arrays.sort(splitters);
		float[] returnvalue = new float[parts];
		returnvalue[0] = splitters[0];
		for (int k = 0;k<parts-2;k++) {
			returnvalue[k+1] = (splitters[k+1] - splitters[k]);
		}
		returnvalue[parts-1] = (totalsum - splitters[parts-2]);
		return returnvalue;
	}
	
	// Load settings from Settings class.
	private void loadSettings(){
		Settings settings = Settings.getInstance();
		tempReductionFactor = settings.getCoolingrate();
		maxAnnealSteps = settings.getMaxIterations();
		swapsPerStep = settings.getGoodSwapsPerTemp();
		trailsPerStep = settings.getTrailsPerTemp();
		tempFactor = settings.getStartTempFactor();
		trim = settings.isSaTrim();
		heel = settings.isSaHeel();
		compatibility = settings.isSaCompatibility();
		strength = settings.isSaStrength();
		usedtanks = settings.isSaUsedtanks();
		trimfactor = settings.getSaTrimfactor();
		heelfactor = settings.getSaHeelfactor();
		compatibilityfactor = settings.getSaCompatibilityfactor();
		strengthfactor = settings.getSaStrengthfactor();
		usedtanksfactor = settings.getSaUsedtanksfactor();
		desiredtrim = settings.getDesiredTrim();
		desiredheel = settings.getDesiredHeel();
		numberOfRestarts = settings.getNumberOfRestarts();
		settings = null;
	}

	/**
	 * Run simulated annealing algorithm.
	 */
	public void run(){
		NumberFormat formatter = NumberFormat.getInstance();
        formatter.setMaximumFractionDigits(2);
        formatter.setMinimumFractionDigits(2);
		
        // Lock interface when not in debug mode
		if(!debug){
	    	MainFrame.getInstance().lock();
	    	ProgressPanel.getInstance().clearProgress();
	 		ProgressPanel.getInstance().startCounter(this);
	 		ProgressPanel.getInstance().addMsg(
					languageResource.getString("saStart"));
	 		Log.getInstance().add(Log.SA, languageResource.getString("saStart"));
	    }
		
		// Start SA algorithm loop: # restarts * maxAnnealSteps
		for(int k= 0; ((k < (maxAnnealSteps*(numberOfRestarts+1))) && start);k++){
			// If new cycle (restart)
			if((k % maxAnnealSteps) == 0){
				// New begin solution + initial temp.
				sol = new Solution();
				temperature = (tempFactor*maxAnnealSteps*
						((usedtanksfactor+trimfactor+
						heelfactor+compatibilityfactor+strengthfactor)));
				if(debug){
					System.out.println("Restart # "+((int) (k/maxAnnealSteps)));
				} else {
					int restarts = (int) (k/maxAnnealSteps);
					ProgressPanel.getInstance().addMsg(
							languageResource.getString("restartNumber")+ ": "+
							formatter.format(restarts));
					Log.getInstance().add(Log.SA, 
							languageResource.getString("restartNumber")+ ": "+
							formatter.format(restarts));
				}
				
				//if first cycle
				if(k == 0){
					bestSol = (Solution) sol.clone();
					bestSolEnergy = sol.getEnergy();
					if(debug){
						System.out.println("Initial temp "+temperature);
					} else {
						ProgressPanel.getInstance().addMsg(
								languageResource.getString("initialTemp")+ ": "+
								formatter.format(temperature));
						Log.getInstance().add(Log.SA, 
								languageResource.getString("initialTemp")+ ": "+
								formatter.format(temperature));
					}
				}
				if(!debug){
					ProgressPanel.getInstance().addMsg(
							languageResource.getString("bestEnergy")+ ": "+
							formatter.format(bestSolEnergy));
					Log.getInstance().add(Log.SA, 
							languageResource.getString("bestEnergy")+ ": "+
							formatter.format(bestSolEnergy));
				}
			}
			// Reset number of swaps for this temperature
			totalSwaps = 0;
			totalNoSwap = 0;
			// Start annealing procedure at this temp
			if(sol.anneal(temperature, trailsPerStep, swapsPerStep)){
				if(debug){
					System.out.println("Temperature "+temperature+ 
							"\ttotalSwaps "+ totalSwaps +
							"\ttotalNoSwaps "+ totalNoSwap);
				} else {
					ProgressPanel.getInstance().addMsg(
							languageResource.getString("currentTemp")+ ": "+
							formatter.format(temperature)+"\t"+
							languageResource.getString("numberSwaps")+ ": "+
							formatter.format(totalSwaps)+"\t"+
							languageResource.getString("numberNoSwaps")+ ": "+
							formatter.format(totalNoSwap));
					Log.getInstance().add(Log.SA, 
							languageResource.getString("currentTemp")+ ": "+
							formatter.format(temperature)+"\t"+
							languageResource.getString("numberSwaps")+ ": "+
							formatter.format(totalSwaps)+"\t"+
							languageResource.getString("numberNoSwaps")+ ": "+
							formatter.format(totalNoSwap));
				}
				// Reduce temp if annealing succesfully found good neighbor
				temperature *= tempReductionFactor;
			} else {
				// Move to next cycle if no good neighbor was found
				int newk = (((int) (k/maxAnnealSteps))+1)*maxAnnealSteps;
				k = newk -1;
			}
			
			solEnergy = sol.getEnergy();
			// Is the energy of the new neighbor better?
			if(solEnergy < bestSolEnergy){
				bestSol = (Solution) sol.clone();
				bestSolEnergy = sol.getEnergy();
			}
			if(!debug){
				ProgressPanel.getInstance().setProgress(k);
			}
		}

		cargoTanks.emptyTanks(true);
		float[] distribution = bestSol.getDistribution();
		int[] cargoid = bestSol.getCargoid();
		for(int k = 0; k < j; k++){
			if(cargoid[k] != -1){
				cargoTanks.get(k).setVolume(distribution[k], 
	    				cargoList.get(cargoid[k]).getDensity());
				cargoTanks.get(k).setCargoid(
						cargoList.get(cargoid[k]).getCargoid());
			}
		}
		printSolution();
	}
	
	/**
	 * Perform step in annealing algorithm.
	 * One temperature cycle will run.
	 */
	public void step(){
		sol.anneal(temperature, trailsPerStep, swapsPerStep);
		temperature *= tempReductionFactor;
		
		solEnergy = sol.getEnergy();
		
		if(solEnergy < bestSolEnergy){
			bestSol = (Solution) sol.clone();
			bestSolEnergy = bestSol.getEnergy();
		}
		
		cargoTanks.emptyTanks(true);
		float[] distribution = bestSol.getDistribution();
		int[] cargoid = bestSol.getCargoid();
		for(int k = 0; k < j; k++){
			if(cargoid[k] != -1){
				cargoTanks.get(k).setVolume(distribution[k], 
	    				cargoList.get(cargoid[k]).getDensity());
				cargoTanks.get(k).setCargoid(
						cargoList.get(cargoid[k]).getCargoid());
			}
		}

		printSolution();
	}
	
	/**
	 * Stop the Thread.
	 */
	public void requestStop(){
		start = false;
	}
	
	/**
	 * Print the output.
	 */
	private void printSolution(){
		if(debug){
			// Output to command line
			if(bestSol.isSlackCargo()){
				System.out.println("Not all cargo distributed");
			}
			if(!bestSol.isCompatible()){
				System.out.println("Compatibility criterion not respected!");
			}
			System.out.println("Final temp "+temperature);
			System.out.println("Trim "+data.getTrim());
			System.out.println("Heel "+data.getHeel());
			System.out.println(bestSol);
			System.out.println("Elapsed: "+(System.currentTimeMillis()-begintime)/1000f);
		} else {
			MainFrame.getInstance().setTabbedPane(2);
	    	MainFrame.getInstance().updateInterface();
	    	if(!step){
	    		MainFrame.getInstance().unlock();
		    	ProgressPanel.getInstance().stopCounter();
	    	}
	    	
	    	// Check if algorithm was canceled or stopped normally
			if(!start) {
				JOptionPane.showMessageDialog(MainFrame.getInstance(),
	    				languageResource.getString("suboptimalMessage"));
			}
	    	if(bestSol.isSlackCargo()){
	    		String message = languageResource.getString("notDistributed")+": \n";
	    		NumberFormat formatter = NumberFormat.getInstance();
		        formatter.setMaximumFractionDigits(2);
		        formatter.setMinimumFractionDigits(2);
	    		for(int k = 0; k < i;k++){
	    			if(bestSol.slackCargo[k] != 0f){
	    				message += cargoList.get(k).getCargoid()+": "
	    				 +formatter.format(bestSol.slackCargo[k])+" m³\n";
	    			}
	    		}
	    		JOptionPane.showMessageDialog(MainFrame.getInstance(),message);
			}
	    	if((!bestSol.isCompatible()) && Settings.getInstance().isSaCompatibility()){
	    		JOptionPane.showMessageDialog(MainFrame.getInstance(),
	    				languageResource.getString("notCompatible"));
			}
		}
	}
	
	/**
	 * Solution object.
	 */
	class Solution implements Cloneable{
		// Three main variables
		float[] distribution;
		int[] cargoid;
		float[] slackCargo;
		
		// Indicates which neighbors have been tried.
		boolean[][] tried;
		
		/**
		 * Constructor. Fills the main variables with initial values
		 * and generates initial solution.
		 */
		public Solution(){
			distribution = new float[j];
			cargoid = new int[j];
			for(int k = 0; k < j; k++){
				distribution[k] = 0;
				cargoid[k] = -1;
			}
			slackCargo = cargoVolumes.clone();
			generateInitial();
		}
		
		/**
		 * Float containing the distribution of the volumes over the tanks.
		 * @return Foating point array containing the volume distribution.
		 */
		public float[] getDistribution(){
			return distribution;
		}
		
		/**
		 * Integer array containing the cargo number for each tank.
		 * Cargo id -1 means that the tank is empty.
		 * @return Integer array containing the cargo id for each tank.
		 */
		public int[] getCargoid(){
			return cargoid;
		}
		
		/**
		 * Check if slack cargo tanks are not empty.
		 * @return Returns true if slack cargo is available, otherwise false.
		 */
		public boolean isSlackCargo(){
			boolean returnvalue = false;
			for(int k = 0; k < i; k++){
				if(slackCargo[k] != 0){
					returnvalue = true;
				}
			}
			return returnvalue;
		}
		
		/**
		 * Returns if compatiblity criteria are respected.
		 * @return True if respected, otherwise false.
		 */
		public boolean isCompatible(){
			boolean returnvalue = true;
			for(int k = 0; k < j; k++){
				if(cargoid[k] != -1){
					if(checkLoadTank(cargoid[k],k)){
						returnvalue = false;
					}
				}
			}
			for(int k1 = 0; k1 < j; k1++){
				for(int k2 = 0; k2 < j; k2++){
					if((segregationCompList.get(k1, k2)) && 
							(cargoid[k1] != -1) && (cargoid[k2] != -1)){
						if(checkLoadLoad(cargoid[k1],cargoid[k2])){
							returnvalue = false;
						}
					}
				}
			}
			return returnvalue;
		}
		
		/**
		 * Start annealing procedure. Find neighbor at current temp.
		 * @param temp Current temperature.
		 * @param nlimit Number of tries at this temperature.
		 * @param glimit Number of maximum good neighbors.
		 * @return Returns true if good neighbor was found, otherwise false.
		 */
		public boolean anneal(float temp, int nlimit, int glimit){	
			// Variables to hold current vars.
			float[] currentDist, currentSlackCargo;
			int[] currentCargoid;
			float currentEnergy, trailEnergy, delta, p, m;
			boolean improvement = false;
			int swaps = 0;
			
			// Annealing loop
			for(int k = 0; k < nlimit; k++){
				// Set current vars.
				currentDist = distribution.clone();
				currentCargoid = cargoid.clone();
				currentSlackCargo = slackCargo.clone();
				currentEnergy = getEnergy();
				
				movetoNeighbor();
				
				trailEnergy = getEnergy();
				delta = currentEnergy - trailEnergy;
				if(delta > 0f){
					// Automatically swap if lower energy
					improvement = true;
					swaps++;
					totalSwaps++;
				} else if(delta < 0f){
					// Swap with chance m if higher energy
					p = (float) Math.random();
					m = (float) Math.exp(delta/temp);
					
					if(p < m){
						improvement = true;
						swaps++;
					} else {
						totalNoSwap++;
						distribution = currentDist;
						cargoid = currentCargoid;
						slackCargo = currentSlackCargo;
					}
				} else {
					 //A neighbor with delta = 0 is only possible when all
					 //neighbors have been tried! Optimum found!
					break;
				}
				
				// Break if enough good neighbors have been found.
				if(swaps >= glimit){
					break;
				}
			}
			
			return improvement;
		}
		
		// Check if tank is empty
		private boolean isEmpty(int k){
			if(cargoid[k] == -1){
				return true;
			} else {
				return false;
			}
		}
		
		// Generate initial solution
		private void generateInitial(){
			// Do while there is slack cargo and not all tanks have been filled.
			while(!checkEveryTankLoaded() && !checkSlackCargo()){
				int maxSlackCargo = 0;

				if(random){
					// Create list of not empty slack Cargo tanks.
					int count = 0;
					int[] notEmpty = new int[i];
					for(int k = 0; k < i; k++){
						if(slackCargo[k] != 0){
							notEmpty[count] = k;
							count++;
						}
					}
					if(count == 0){
						break;
					}
					// Pick random slack cargo from list.
					maxSlackCargo = notEmpty[(int) (Math.random()*count)];
				} else {
					// Get largest slack cargo volume available.
					maxSlackCargo = getMax(slackCargo);
				}
				
				// Create list of empty cargo tanks.
				int[] Empty = new int[j];
				int count = 0;
				for(int k = 0; k < j; k++){
					if(cargoid[k] == -1){
						Empty[count] = k;
						count++;
					}
				}
				if(count == 0){
					break;
				}
				// Pick random tank from list
				int randomTank = Empty[(int) (Math.random()*count)];
				
				// If volume is larger then tank, then the tank will be
				// filled completely and remainder will stay in slack tank.
				// Otherwise the whole slack cargo volume will be moved to
				// tank.
				if(tankVolumes[randomTank] < slackCargo[maxSlackCargo]){
					distribution[randomTank] = tankVolumes[randomTank];
					cargoid[randomTank] = maxSlackCargo;
					slackCargo[maxSlackCargo] -= tankVolumes[randomTank];
				} else {
					distribution[randomTank] = slackCargo[maxSlackCargo];
					cargoid[randomTank] = maxSlackCargo;
					slackCargo[maxSlackCargo] = 0;
				}
			}
		}
		
		/**
		 * Get energy level. Indicates how good a solution is.
		 * A lower energy level is better.
		 * @return Floating point value containing energy level.
		 */
		public float getEnergy(){
			float returnvalue = 0f;
			// Try to get empty slack cargo tanks
			for(int k = 0; k< i; k++){
				if(slackCargo[k] != 0){
					returnvalue += (150*(slackCargo[k]/cargoVolumes[k]));
				}
			}
			
			// Minimize number of tanks used.
			if(usedtanks){
				for(int k = 0; k < j; k++){
					if((cargoid[k] != -1) && (distribution[k] != tankVolumes[k])){
						returnvalue += (usedtanksfactor*
								((tankVolumes[k] - distribution[k])/tankVolumes[k]));
					} else if(distribution[k] == tankVolumes[k]){
						returnvalue += (usedtanksfactor/j);
					}
				}
			}
			
			// To check trim or heel. The current solution has to be loaded into
			// the ship.
			if(trim || heel){
				cargoTanks.emptyTanks(true);
				for(int k = 0; k < j; k++){
					if(cargoid[k] != -1){
						cargoTanks.get(k).setVolume(distribution[k], 
			    				cargoList.get(cargoid[k]).getDensity());
					}
				}
			}
			
			// Get desired trim.
			if(trim){
				returnvalue += (trimfactor*10*Math.abs(desiredtrim-data.getTrim()));
			}
			
			// Get desired heel.
			if(heel){
				returnvalue += (heelfactor*10*Math.abs(desiredheel-data.getHeel()));
			}
			
			// See how many compatibility criteria are not respected.
			if(compatibility){
				for(int k = 0; k < j; k++){
					if(cargoid[k] != -1){
						if(checkLoadTank(cargoid[k],k)){
							returnvalue += compatibilityfactor;
						}
					}
				}
				for(int k1 = 0; k1 < j; k1++){
					for(int k2 = 0; k2 < j; k2++){
						if((segregationCompList.get(k1, k2)) && 
								(cargoid[k1] != -1) && (cargoid[k2] != -1)){
							if(checkLoadLoad(cargoid[k1],cargoid[k2])){
								returnvalue += (compatibilityfactor/2);
							}
						}
					}
				}
			}
			
			// Optimize hull stress
			if(strength){
				for(int k = 0; k < z; k++){
					float downward = 0f;
					for(int k2 = 0; k2 < j; k2++){
						if(cargoid[k2] != -1){
							downward += (sectionList.get(k).getTankfactors(k2) * 
						 	distribution[k2]*cargoList.get(cargoid[k2]).getDensity());
						}
					}
					returnvalue += (Math.abs(downward - sectionList.get(k).getBuoymisc())
							* (strengthfactor/(z*sectionList.get(k).getMaxforce())));
				}
			}
			return returnvalue;
		}
		
		/**
		 * Check if load is compatible with tank.
		 * @param i1 Integer indicating load.
		 * @param j1 Integer indicating tank.
		 * @return Returns true if incompatible, otherwise false;
		 */
		public boolean checkLoadTank(int i1, int j1){
			return tankCompList.get(i1, j1);
		}
		
		/**
		 * Check if load is compatible with load.
		 * @param i1 Integer indicating first load.
		 * @param i2 Integer indicating secodn load.
		 * @return Returns true if incompatible, otherwise false;
		 */
		public boolean checkLoadLoad(int i1, int i2){
			return cargoCompList.get(i1, i2);
		}
		
		/**
		 * Move to neighboring solution.
		 */
		public void movetoNeighbor(){
			float currentEnergy = getEnergy();
			float newEnergy = currentEnergy;
			tried = createEmptyTried();
			
			// Do while energy is not the same. A solution with the same energy
			// is almost certain the same solution.
			while(currentEnergy == newEnergy){
				// Get two appropriate candidates
				int[] rands = getCandidates();

				int rand1 = rands[0];
				int rand2 = rands[1];
				
				//Switch the volumes of the two tanks
				switchTanks(rand1,rand2);
				
				// Remove slack tanks and distribute cargo from slack cargo tanks
				boolean remslack = true;
				boolean distslack = true;
				while(remslack || distslack){
					remslack = removeSlackTanks();
					distslack = distributeSlack();
				}
				
				newEnergy = getEnergy();
				// Add solutions to tried list when they have the same energy.
				if(currentEnergy == newEnergy){
					tried[rand1][rand2] = true;
					tried[rand2][rand1] = true;
					// Break out of loop when all solutions have been tried.
					if(triedAll()){
						break;
					}
				}
			}
		}
		
		// Create new list with tried candidates
		private boolean[][] createEmptyTried(){
			boolean[][] returnvalue = new boolean[j][j];
			for(int k1 = 0; k1 < j; k1++){
				for(int k2 = 0; k2 < j; k2++){
					if((k1 == k2) || (isEmpty(k1) && isEmpty(k2))){
						returnvalue[k1][k2] = true;
					} else {
						returnvalue[k1][k2] = false;
					}
					
				}
			}
			return returnvalue;
		}

		// Get two appropriate random candidates
		private int[] getCandidates(){
			// Create list of appropriate first random candidate
			int[] posrand1 = new int[j];
			int count = 0;
			for(int k1 = 0; k1 < j; k1++){
				boolean nottried = true;
				for(int k2 = 0; k2< j;k2++){
					nottried = tried[k1][k2];
					if(!nottried){
						break;
					}
				}
				if(!nottried){
					posrand1[count] = k1;
					count++;
				}
			}
			// Select first random from this list
			int rand1 = posrand1[(int) (Math.random()*count)];
			
			// Create list of appropriate second random candidate
			count = 0;
			int[] posrand2 = new int[j];
			for(int k = 0; k < j;k++){
				if(!tried[rand1][k]){
					if((k != rand1) || (isEmpty(rand1) && isEmpty(k))){
						posrand2[count] = k;
						count++;
					}
				}
			}
			
			// Select second random from this list
			int rand2 = posrand2[(int) (Math.random()*count)];
			
			int[] returnvalue = new int[2];
			returnvalue[0] = rand1;
			returnvalue[1] = rand2;
			
			return returnvalue;
		}
		
		// Check if all neighbors have been tried
		private boolean triedAll(){
			boolean returnvalue = true;
			outer: for(int k1 = 1; k1 < j; k1++){
				for(int k2 = 0; k2<k1;k2++){
					if(!tried[k1][k2]){
						returnvalue = false;
						break outer;
					}
				}
			}
			return returnvalue;
		}
		
		// Switch two tanks. If volume is to large it will be transferred to
		// slack cargo tank.
		private void switchTanks(int j1, int j2){
			float tempdist = distribution[j1];
			int tempcargoid = cargoid[j1];
			distribution[j1] = distribution[j2];
			cargoid[j1] = cargoid[j2];
			distribution[j2] = tempdist;
			cargoid[j2] = tempcargoid;
			if(distribution[j1] > tankVolumes[j1]){
				slackCargo[cargoid[j1]] += (distribution[j1] - tankVolumes[j1]);
				distribution[j1] = tankVolumes[j1];
			}
			if(distribution[j2] > tankVolumes[j2]){
				slackCargo[cargoid[j2]] += (distribution[j2] - tankVolumes[j2]);
				distribution[j2] = tankVolumes[j2];
			}
		}
		
		// Procedure to distribute loads from slack cargo tanks to regular tanks.
		private boolean distributeSlack(){
			boolean returnvalue = false;
			// Distribute slack cargo over tanks that are not full
			for(int k = 0; k < i;k++){
			if(slackCargo[k] !=0){
				for(int k2 = 0; k2 < j;k2++){
					if((cargoid[k2] == k) && 
							(distribution[k2] != tankVolumes[k2])){
						if((tankVolumes[k2] - distribution[k2])<=slackCargo[k]){
							slackCargo[k] -= (tankVolumes[k2] - distribution[k2]);
							distribution[k2] = tankVolumes[k2];
							returnvalue = true;
						} else {
							distribution[k2] += slackCargo[k];
							slackCargo[k] = 0;
							returnvalue = true;
							break;
						}
					}
				}
			}
			}
			// Dirstribute slack cargo over empty tanks
			while(!checkEveryTankLoaded() && !checkSlackCargo()){
				int maxSlackCargo = 0;
				int appropriateTank = 0;
				
				// When not random an empty tank is chosen that bests fits
				// the distributed volume
				if(random){
					int count = 0;
					int[] notEmpty = new int[i];
					for(int k = 0; k < i; k++){
						if(slackCargo[k] != 0){
							notEmpty[count] = k;
							count++;
						}
					}
					if(count == 0){
						break;
					}
					maxSlackCargo = notEmpty[(int) (Math.random()*count)];
					int[] Empty = new int[j];
					count = 0;
					for(int k = 0; k < j; k++){
						if(cargoid[k] == -1){
							Empty[count] = k;
							count++;
						}
					}
					if(count == 0){
						break;
					}
					appropriateTank = Empty[(int) (Math.random()*count)];
				} else {
					maxSlackCargo = getMax(slackCargo);
					appropriateTank = getAppropriateEmptyTank(slackCargo[maxSlackCargo]);
				}
				if(slackCargo[maxSlackCargo] > tankVolumes[appropriateTank]){
					distribution[appropriateTank] = tankVolumes[appropriateTank];
					slackCargo[maxSlackCargo] -= tankVolumes[appropriateTank];
					cargoid[appropriateTank] = maxSlackCargo;
					returnvalue = true;
				} else {
					distribution[appropriateTank] += slackCargo[maxSlackCargo];
					slackCargo[maxSlackCargo] = 0;
					cargoid[appropriateTank] = maxSlackCargo;
					returnvalue = true;
				}
			}
			return returnvalue;
		}
		
		// Get the appropriate empty tank for a vol that has to be distributed
		private int getAppropriateEmptyTank(float vol){
			int[] emptyTanks = getCargoIdTanks(-1, -1);
			int returnvalue = emptyTanks[(int) (Math.random()*emptyTanks.length)];
			float currentMin = (tankVolumes[returnvalue]-vol);
			for(int k = 0; k < j; k++){
				if((cargoid[k] == -1) && (Math.abs(tankVolumes[k]-vol)<currentMin)){
					returnvalue = k;
					currentMin = Math.abs(tankVolumes[k]-vol);
				}
			}
			return returnvalue;
		}
		
		// Procedure that combines to slack tanks with the same cargo
		private boolean removeSlackTanks(){
			boolean returnvalue = false;
			for(int k =0; k < j;k++){
				if((distribution[k] != tankVolumes[k]) && (distribution[k] != 0f)){
					int[] sameid = getCargoIdTanks(k,cargoid[k]);
					if(sameid != null){
						int sec = 0;
						// When not random an appropriate second tank will be chosen
						if(random){
							sec = (int) (Math.random()*sameid.length);
						} else {
							sec = getAppropriateCombTank(k, sameid);
						}
						combineTanks(k,sameid[sec]);
						returnvalue = true;
					}
					
					
				}
			}
			return returnvalue;
		}
		
		// Appropriate second tank when two tanks with the same cargo are combined
		private int getAppropriateCombTank(int j1, int[] sameid){
			int returnvalue = (int) (Math.random()*sameid.length);
			int j2 = sameid[returnvalue];
			float bestcomb = 0;
			if(((tankVolumes[j1]-distribution[j1])>=distribution[j2]) &&
					((tankVolumes[j2]-distribution[j2])>=distribution[j1])){
				if((tankVolumes[j1]-distribution[j1]-distribution[j2])>
				(tankVolumes[j2]-distribution[j2]-distribution[j1])){
					bestcomb = tankVolumes[j2]-distribution[j2]-distribution[j1];
				} else {
					bestcomb = tankVolumes[j1]-distribution[j1]-distribution[j2];
				}
			} else if ((tankVolumes[j1]-distribution[j1])>=distribution[j2]){
				bestcomb = tankVolumes[j1] - distribution[j1] - distribution[j2];
			} else if((tankVolumes[j2]-distribution[j2])>=distribution[j1]){
				bestcomb = tankVolumes[j2] - distribution[j2] - distribution[j1];;
			} else {
				bestcomb = tankVolumes[j1] + tankVolumes[j2] - 
									distribution[j2] - distribution[j1];
			}
			float comb = 0;
			for(int k = 0; k< sameid.length;k++){
				j2 = sameid[k];
				if(((tankVolumes[j1]-distribution[j1])>=distribution[j2]) &&
						((tankVolumes[j2]-distribution[j2])>=distribution[j1])){
					if((tankVolumes[j1]-distribution[j1]-distribution[j2])>
					(tankVolumes[j2]-distribution[j2]-distribution[j1])){
						comb = tankVolumes[j2]-distribution[j2]-distribution[j1];
					} else {
						comb = tankVolumes[j1]-distribution[j1]-distribution[j2];
					}
				} else if ((tankVolumes[j1]-distribution[j1])>=distribution[j2]){
					comb = tankVolumes[j1] - distribution[j1] - distribution[j2];
				} else if((tankVolumes[j2]-distribution[j2])>=distribution[j1]){
					comb = tankVolumes[j2] - distribution[j2] - distribution[j1];;
				} else {
					comb = tankVolumes[j1] + tankVolumes[j2] - 
										distribution[j2] - distribution[j1];
				}
				if(comb < bestcomb){
					returnvalue = k;
					bestcomb = comb;
				}
			}
			return returnvalue;
		}
		
		// Combine two tanks
		private void combineTanks(int j1, int j2){
			if(((tankVolumes[j1]-distribution[j1])>=distribution[j2]) &&
					((tankVolumes[j2]-distribution[j2])>=distribution[j1])){
				if((tankVolumes[j1]-distribution[j1]-distribution[j2])>
					(tankVolumes[j2]-distribution[j2]-distribution[j1])){
					distribution[j2] += distribution[j1];
					distribution[j1] = 0;
					cargoid[j1] = -1;
					return;
				} else {
					distribution[j1] += distribution[j2];
					distribution[j2] = 0;
					cargoid[j2] = -1;
					return;
				}
			} else if((tankVolumes[j1]-distribution[j1])>=distribution[j2]){
				distribution[j1] += distribution[j2];
				distribution[j2] = 0;
				cargoid[j2] = -1;
				return;
			} else if((tankVolumes[j2]-distribution[j2])>=distribution[j1]){
				distribution[j2] += distribution[j1];
				distribution[j1] = 0;
				cargoid[j1] = -1;
				return;
			} else if(tankVolumes[j1] <= tankVolumes[j2]){
				distribution[j1] -= (tankVolumes[j2]-distribution[j2]);
				distribution[j2] = tankVolumes[j2];
				return;
			} else {
				distribution[j2] -= (tankVolumes[j1]-distribution[j1]);
				distribution[j1] = tankVolumes[j1];
				return;
			}
		}
		
		// Get list of tanks with same cargo id. Original tank not included.
		private int[] getCargoIdTanks(int tank, int id){
			int count = 0;
			for(int k =0; k < j;k++){
				if((cargoid[k] == id) && (distribution[k] != tankVolumes[k]) &&
						k != tank){
					count++;
				}
			}
			if(count == 0){
				return null;
			}
			int[] returnvalue = new int[count];
			count = 0;
			for(int k = 0; k < j; k++){
				if((cargoid[k] == id) && (distribution[k] != tankVolumes[k])&&
						k != tank){
					returnvalue[count] = k;
					count++;
				}
			}
			return returnvalue;
		}
		
		/**
		 * Overwritten method  to get correct output to command line.
		 * Used in debug procedures.
		 */
		public String toString(){
			String returnvalue = "";
			NumberFormat formatter = NumberFormat.getInstance();
	        formatter.setMaximumFractionDigits(0);
	        formatter.setMinimumFractionDigits(0);
			for(int k = 0; k < j; k++){
				returnvalue += (formatter.format(distribution[k]) + "\t");
			}
			returnvalue += "\n";
			for(int k = 0; k < j; k++){
				returnvalue += (formatter.format(tankVolumes[k]) + "\t");
			}
			returnvalue += "\n";
			for(int k = 0; k < j; k++){
				returnvalue += (formatter.format(cargoid[k]) + "\t");
			}
			returnvalue += "\n";
			for(int k = 0; k < i; k++){
				returnvalue += (formatter.format(slackCargo[k]) + "\t");
			}
			returnvalue += "\n";
			for(int k = 0; k < i; k++){
				returnvalue += (formatter.format(cargoVolumes[k]) + "\t");
			}
			returnvalue += "\n";
			formatter.setMaximumFractionDigits(3);
	        formatter.setMinimumFractionDigits(3);
			returnvalue += ("Energy "+formatter.format(getEnergy()));
			
			return returnvalue;
		}
		
		// Get the largest vol in array
		private int getMax(float[]array){
			float currentmax = 0f;
			int returnvalue = -1;
			for(int k = 0; k < array.length; k++){
				if(array[k]>= currentmax){
					returnvalue = k;
					currentmax = array[k];
				}
			}
			return returnvalue;
		}
		
		// Check if there is slack cargo
		private boolean checkSlackCargo(){
			float total = 0f;
			for(int k = 0; k< slackCargo.length;k++){
				total += slackCargo[k];
			}
			if(total == 0f){
				return true;
			} else {
				return false;
			}
		}
		
		// Check if every tank is loaded
		private boolean checkEveryTankLoaded(){
			boolean returnvalue = true;
			for(int k = 0; k< distribution.length; k++){
				if(cargoid[k] == -1){
					returnvalue = false;
				}
			}
			return returnvalue;
		}
		
		/**
		 * Overwritten method used to get a clone of the solution.
		 */
		public Object clone(){
			try {
				Object clone = super.clone();
				((Solution) clone).distribution = distribution.clone();
				((Solution) clone).slackCargo = slackCargo.clone();
				((Solution) clone).cargoid = cargoid.clone();
				return clone;
			} catch (CloneNotSupportedException e){
				throw new InternalError();
			}
		}
	}
}
