//: /be/optiloading/settings/GeneticPanel.java
/*
 * This work is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2008 Tim Pintens. All rights reserved.
 */

package be.optiloading.settings;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.text.NumberFormat;
import java.util.ResourceBundle;

import javax.swing.BoxLayout;
import javax.swing.JComboBox;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSeparator;
import javax.swing.JSlider;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingConstants;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * Panel for genetic settings.
 * 
 * @author Tim Pintens, info@optiloading.be
 * @author <a target="_blank"
 *         href="http://www.optiloading.be">www.optiloading.be</a>
 */

@SuppressWarnings("serial")
class GeneticPanel extends JPanel {
	JComboBox creationbox;
	JComboBox selectionbox;
	JComboBox crossoverbox;
	JComboBox mutationbox;
	JFormattedTextField mutratefield;
	JFormattedTextField crossratefield;
	JFormattedTextField popsizefield;
	JFormattedTextField eliteratefield;
	JSlider mutrate;
	JSlider crossrate;
	JSlider eliterate;
	JSlider popsize;
	JSpinner restarts;
	JSpinner optimum;
	
	/**
	 * Constructor to build panel.
	 */
	public GeneticPanel(){
		ResourceBundle languageResource = Settings.languageResource;
		
		// Mutation rate
		JPanel mutratepaneltop = new JPanel();
		mutratepaneltop.setLayout(new BorderLayout());
		mutratefield = new JFormattedTextField(NumberFormat.getInstance());
		mutratefield
				.setToolTipText(languageResource.getString("mutrateTooltip"));
		mutratefield.setBackground(new Color(236, 236, 236));
		mutratefield.setHorizontalAlignment(JTextField.RIGHT);
		mutratefield.setValue(new Float(0.10));
		mutratefield.setEditable(false);
		mutratefield.setColumns(6);
		mutrate = new JSlider(0, 100, 10);
		mutrate.setToolTipText(languageResource.getString("mutrateTooltip"));
		mutrate.addChangeListener(new MutRateListener());
		mutratepaneltop.add(BorderLayout.WEST, new JLabel(languageResource.
				getString("mutrateLabel")+":"));
		mutratepaneltop.add(BorderLayout.EAST, mutratefield);
		JPanel mutratepanelbottom = new JPanel();
		mutratepanelbottom.setLayout(new BorderLayout());
		mutratepanelbottom.add(BorderLayout.NORTH, mutrate);
		JPanel mutratepanel = new JPanel();
		mutratepanel.setLayout(new BorderLayout());
		mutratepanel.add(BorderLayout.NORTH, mutratepaneltop);
		mutratepanel.add(BorderLayout.SOUTH, mutratepanelbottom);

		// Crossover rate
		JPanel crossratepaneltop = new JPanel();
		crossratepaneltop.setLayout(new BorderLayout());
		crossratefield = new JFormattedTextField(NumberFormat.getInstance());
		crossratefield
				.setToolTipText(languageResource.getString("crossrateTooltip"));
		crossratefield.setBackground(new Color(236, 236, 236));
		crossratefield.setHorizontalAlignment(JTextField.RIGHT);
		crossratefield.setValue(new Float(0.70));
		crossratefield.setEditable(false);
		crossratefield.setColumns(6);
		crossrate = new JSlider(0, 100, 70);
		crossrate
				.setToolTipText(languageResource.getString("crossrateTooltip"));
		crossrate.addChangeListener(new CrossRateListener());
		crossratepaneltop.add(BorderLayout.WEST, new JLabel(languageResource.
				getString("crossrateLabel")+":"));
		crossratepaneltop.add(BorderLayout.EAST, crossratefield);
		JPanel crossratepanelbottom = new JPanel();
		crossratepanelbottom.setLayout(new BorderLayout());
		crossratepanelbottom.add(BorderLayout.NORTH, crossrate);
		JPanel crossratepanel = new JPanel();
		crossratepanel.setLayout(new BorderLayout());
		crossratepanel.add(BorderLayout.NORTH, crossratepaneltop);
		crossratepanel.add(BorderLayout.SOUTH, crossratepanelbottom);

		// Population Size
		JPanel popsizepaneltop = new JPanel();
		popsizepaneltop.setLayout(new BorderLayout());
		popsizefield = new JFormattedTextField(NumberFormat
				.getIntegerInstance());
		popsizefield
				.setToolTipText(languageResource.getString("poprateTooltip"));
		popsizefield.setBackground(new Color(236, 236, 236));
		popsizefield.setHorizontalAlignment(JTextField.RIGHT);
		popsizefield.setValue(new Integer(500));
		popsizefield.setEditable(false);
		popsizefield.setColumns(6);
		popsize = new JSlider(2, 1000, 500);
		popsize
				.setToolTipText(languageResource.getString("poprateTooltip"));
		popsize.addChangeListener(new PopSizeListener());
		popsizepaneltop.add(BorderLayout.WEST, new JLabel(languageResource.
				getString("poprateLabel")+":"));
		popsizepaneltop.add(BorderLayout.EAST, popsizefield);
		JPanel popsizepanelbottom = new JPanel();
		popsizepanelbottom.setLayout(new BorderLayout());
		popsizepanelbottom.add(BorderLayout.NORTH, popsize);
		JPanel popsizepanel = new JPanel();
		popsizepanel.setLayout(new BorderLayout());
		popsizepanel.add(BorderLayout.NORTH, popsizepaneltop);
		popsizepanel.add(BorderLayout.SOUTH, popsizepanelbottom);

		// Elitism rate
		JPanel eliteratepaneltop = new JPanel();
		eliteratepaneltop.setLayout(new BorderLayout());
		eliteratefield = new JFormattedTextField(NumberFormat.getInstance());
		eliteratefield
				.setToolTipText(languageResource.getString("eliterateTooltip"));
		eliteratefield.setBackground(new Color(236, 236, 236));
		eliteratefield.setHorizontalAlignment(JTextField.RIGHT);
		eliteratefield.setValue(new Float(0.02));
		eliteratefield.setEditable(false);
		eliteratefield.setColumns(6);
		eliterate = new JSlider(0, 50, 2);
		eliterate.setToolTipText(languageResource.getString("eliterateTooltip"));
		eliterate.addChangeListener(new EliteRateListener());
		eliteratepaneltop.add(BorderLayout.WEST, new JLabel(languageResource.
				getString("eliterateLabel")+":"));
		eliteratepaneltop.add(BorderLayout.EAST, eliteratefield);
		JPanel eliteratepanelbottom = new JPanel();
		eliteratepanelbottom.setLayout(new BorderLayout());
		eliteratepanelbottom.add(BorderLayout.NORTH, eliterate);
		JPanel eliteratepanel = new JPanel();
		eliteratepanel.setLayout(new BorderLayout());
		eliteratepanel.add(BorderLayout.NORTH, eliteratepaneltop);
		eliteratepanel.add(BorderLayout.SOUTH, eliteratepanelbottom);

		// Creation method
		JPanel creationpanel = new JPanel();
		creationpanel.setLayout(new BorderLayout());
		creationpanel.add(BorderLayout.WEST, new JLabel(languageResource.
				getString("creationLabel")+":"));
		String[] creationlist = {languageResource.getString("creationRandom"),
				languageResource.getString("creationFull")};
		creationbox = new JComboBox(creationlist);
		creationbox.setSelectedIndex(1);
		creationbox.setToolTipText(languageResource.getString("creationTooltip"));
		creationpanel.add(BorderLayout.EAST, creationbox);

		// Selection method
		JPanel selectionpanel = new JPanel();
		selectionpanel.setLayout(new BorderLayout());
		selectionpanel.add(BorderLayout.WEST, new JLabel(languageResource.
				getString("selectionLabel")+":"));
		String[] selectionlist = { languageResource.getString("selectionRank"), 
				languageResource.getString("selectionRoulette")};
		selectionbox = new JComboBox(selectionlist);
		selectionbox.setToolTipText(languageResource.getString("selectionTooltip"));
		selectionpanel.add(BorderLayout.EAST, selectionbox);

		// Crossover method
		JPanel crossoverpanel = new JPanel();
		crossoverpanel.setLayout(new BorderLayout());
		crossoverpanel.add(BorderLayout.WEST, new JLabel(languageResource.
				getString("crossoverLabel")+":"));
		String[] crossoverlist = { languageResource.getString("crossoverMean"), 
				languageResource.getString("crossoverOnePoint"), 
				languageResource.getString("crossoverTwoPoint"), 
				languageResource.getString("crossoverUniform")};
		crossoverbox = new JComboBox(crossoverlist);
		crossoverbox.setSelectedIndex(3);
		crossoverbox.setToolTipText(languageResource.getString("crossoverTooltip"));
		crossoverpanel.add(BorderLayout.EAST, crossoverbox);

		// Mutation method
		JPanel mutationpanel = new JPanel();
		mutationpanel.setLayout(new BorderLayout());
		mutationpanel.add(BorderLayout.WEST, new JLabel(languageResource.
				getString("mutationLabel")+":"));
		mutationpanel.add(BorderLayout.CENTER, new JLabel("     "));
		String[] mutationlist = { languageResource.getString("mutationSwitch"), 
				languageResource.getString("mutationAddSubtract") };
		mutationbox = new JComboBox(mutationlist);
		mutationbox.setToolTipText(languageResource.getString("mutationTooltip"));
		mutationpanel.add(BorderLayout.EAST, mutationbox);

		// Genetic rates
		JPanel geneticrates = new JPanel();
		geneticrates.setBorder(new EmptyBorder(3, 5, 3, 17));
		geneticrates.setLayout(new BoxLayout(geneticrates, BoxLayout.Y_AXIS));
		geneticrates.add(mutratepanel);
		geneticrates.add(crossratepanel);
		geneticrates.add(popsizepanel);
		geneticrates.add(eliteratepanel);

		// Genetic methods
		JPanel geneticmethods = new JPanel();
		geneticmethods.setBorder(new EmptyBorder(3, 10, 3, 5));
		geneticmethods
				.setLayout(new BoxLayout(geneticmethods, BoxLayout.Y_AXIS));
		creationbox.setPreferredSize(mutationbox.getPreferredSize());
		crossoverbox.setPreferredSize(mutationbox.getPreferredSize());
		selectionbox.setPreferredSize(mutationbox.getPreferredSize());
		creationpanel.setPreferredSize(mutationpanel.getPreferredSize());
		selectionpanel.setPreferredSize(mutationpanel.getPreferredSize());
		crossoverpanel.setPreferredSize(mutationpanel.getPreferredSize());
		JPanel method1 = new JPanel(new GridBagLayout());
		method1.add(creationpanel, new GridBagConstraints());
		geneticmethods.add(method1);
		JPanel method2 = new JPanel(new GridBagLayout());
		method2.add(selectionpanel, new GridBagConstraints());
		geneticmethods.add(method2);
		JPanel method3 = new JPanel(new GridBagLayout());
		method3.add(crossoverpanel, new GridBagConstraints());
		geneticmethods.add(method3);
		JPanel method4 = new JPanel(new GridBagLayout());
		method4.add(mutationpanel, new GridBagConstraints());
		geneticmethods.add(method4);
		
		// Add to genetic settings panel
		JPanel geneticpanel = new JPanel();
		geneticpanel.setLayout(new BoxLayout(geneticpanel,BoxLayout.X_AXIS));
		geneticpanel.setBorder(new CompoundBorder(new TitledBorder(
				new EtchedBorder(), languageResource.getString("geneticRatesMethods")), 
				new EmptyBorder(1, 5, 3, 5)));
		geneticpanel.add(geneticrates);
		geneticpanel.add(new JSeparator(SwingConstants.VERTICAL));
		geneticpanel.add(geneticmethods);
		
		// Stop condition
		Dimension shortField = new Dimension( 80, 22 );
		GridBagConstraints c = new GridBagConstraints();
		c.insets = new Insets( 2, 2, 2, 2 );
		c.anchor = GridBagConstraints.WEST;
		c.ipady = 5;
		
		JPanel condition = new JPanel();
		condition.setLayout(new GridBagLayout());
		
		//Optimum
		JLabel labeloptimum = new JLabel(languageResource.
				getString("optimumLabel")+":     ");
		condition.add(labeloptimum,c);
		optimum = new JSpinner(new SpinnerNumberModel(25, 5, 1000, 1));
		optimum.setPreferredSize(shortField);
		c.gridx = 1;
	    c.gridwidth = 3; 
	    condition.add(optimum, c);
		
	    //Restarts
		JLabel labelrestarts = new JLabel(languageResource.
				getString("restartsLabel")+":");
		c.gridwidth = 1;
	    c.gridx = 0;
	    c.gridy = 1;
		condition.add(labelrestarts,c);
		restarts = new JSpinner(new SpinnerNumberModel(5, 1, 100, 1));
		restarts.setPreferredSize(shortField);
		c.gridx = 1;
	    c.gridwidth = 3; 
	    condition.add(restarts, c);
		
	    JPanel conditionmain = new JPanel(new BorderLayout());
	    conditionmain.setBorder(new CompoundBorder(new TitledBorder(
				new EtchedBorder(), languageResource.getString("stopCondition")), 
				new EmptyBorder(1, 5, 3, 5)));
		conditionmain.add(BorderLayout.WEST,condition);
	    
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		add(conditionmain);
		add(geneticpanel);
	}
	
	// ChangeListener that changes the value of the field when the slider is
	// moved
	class MutRateListener implements ChangeListener {
		public void stateChanged(ChangeEvent e) {
			JSlider source = (JSlider) e.getSource();
			float floatvalue = (source.getValue() / 100f);
			mutratefield.setValue(floatvalue);
		}
	}

	// ChangeListener that changes the value of the field when the slider is
	// moved
	class CrossRateListener implements ChangeListener {
		public void stateChanged(ChangeEvent e) {
			JSlider source = (JSlider) e.getSource();
			float floatvalue = (source.getValue() / 100f);
			crossratefield.setValue(floatvalue);
		}
	}

	// ChangeListener that changes the value of the field when the slider is
	// moved
	class PopSizeListener implements ChangeListener {
		public void stateChanged(ChangeEvent e) {
			JSlider source = (JSlider) e.getSource();
			int nvalue;
			if (source.getValue() % 2 != 0) {
				nvalue = source.getValue() + 1;
			} else {
				nvalue = source.getValue();
			}
			popsizefield.setValue(nvalue);
		}
	}

	// ChangeListener that changes the value of the field when the slider is
	// moved
	class EliteRateListener implements ChangeListener {
		public void stateChanged(ChangeEvent e) {
			JSlider source = (JSlider) e.getSource();
			float floatvalue = (source.getValue() / 100f);
			eliteratefield.setValue(floatvalue);
		}
	}
}
