//: /be/optiloading/settings/LpPanel.java
/*
 * This work is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2008 Tim Pintens. All rights reserved.
 */

package be.optiloading.settings;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.text.DecimalFormat;
import java.text.ParseException;
import java.util.ResourceBundle;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFormattedTextField;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingConstants;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;
import javax.swing.text.DefaultFormatterFactory;
import javax.swing.text.NumberFormatter;

/**
 * Panel for lineair programming settings.
 * 
 * @author Tim Pintens, info@optiloading.be
 * @author <a target="_blank"
 *         href="http://www.optiloading.be">www.optiloading.be</a>
 */

@SuppressWarnings("serial")
class LpPanel extends JPanel {
	//Objective
	JCheckBox usedtanks;
	JCheckBox strength;
	
	//Constraints
	JCheckBox heel;
	JCheckBox compatibility;
	JCheckBox stability;
	JCheckBox trim;
	JFormattedTextField stabfield;
	JFormattedTextField trimfield;
	JFormattedTextField usedtanksfield;
	JFormattedTextField strengthfield;
	
	//Solver
	JComboBox solverbox;
	
	private ResourceBundle languageResource;
	private JSpinner stabspinner;
	private JSpinner trimspinner;
	private JSpinner usedtanksspinner;
	private JSpinner strengthspinner;
	
	/**
	 * Constructor to build panel.
	 */
	public LpPanel(){
		languageResource = Settings.languageResource;
		
		//	Criteria Panel
		JPanel criteria = new JPanel();
		GridBagConstraints c = new GridBagConstraints();
		c.insets = new Insets( 2, 2, 2, 2 );
		c.anchor = GridBagConstraints.WEST;
		criteria.setLayout(new GridBagLayout());
		criteria.setBorder(new CompoundBorder(new TitledBorder(
				new EtchedBorder(), languageResource.getString("criteriaTitle")),
				new EmptyBorder(1, 5, 3, 5)));
		
		// Used tanks
		usedtanks = new JCheckBox(languageResource.getString("criteriaUsedtanks"), null, true);
		usedtanks.addItemListener(new CriteriaListener());
		usedtanks.setToolTipText(languageResource.
				getString("criteriaUsedtanksTooltip"));
		usedtanks.addItemListener(new ItemListener(){
			public void itemStateChanged(ItemEvent e) {
				if(usedtanks.isSelected()){
					usedtanksfield.setEnabled(true);
					usedtanksspinner.setEnabled(true);
				} else {
					usedtanksfield.setEnabled(false);
					usedtanksspinner.setEnabled(false);
				}
			}
		});
		criteria.add(usedtanks,c);
		SpinnerNumberModel usedtanksmod = new SpinnerNumberModel(1,1,100,1);
		usedtanksspinner = new JSpinner(usedtanksmod);
		usedtanksspinner.setEditor(new JSpinner.NumberEditor(usedtanksspinner, "0"));
		usedtanksspinner.setToolTipText(languageResource.getString("usedtankscriTooltip"));
		usedtanksfield = ((JSpinner.DefaultEditor) usedtanksspinner.getEditor()).getTextField();
		usedtanksfield.setFont(new Font("Dialog",Font.PLAIN,12));
		usedtanksspinner.setPreferredSize(new Dimension( 80, 22 ));
		c.gridx = 1;
		c.gridwidth = 3;
		criteria.add(usedtanksspinner, c);
		
		// Strength
		strength = new JCheckBox(languageResource.getString("criteriaStress"), null, true);
		strength.addItemListener(new CriteriaListener());
		strength.setToolTipText(languageResource.
				getString("criteriaStresTooltip"));
		strength.addItemListener(new ItemListener(){
			public void itemStateChanged(ItemEvent e) {
				if(strength.isSelected()){
					strengthfield.setEnabled(true);
					strengthspinner.setEnabled(true);
				} else {
					strengthfield.setEnabled(false);
					strengthspinner.setEnabled(false);
				}
			}
		});
		c.gridwidth = 1;
	    c.gridx = 0;
	    c.gridy = 1;
	    criteria.add(strength,c);
		SpinnerNumberModel strengthmod = new SpinnerNumberModel(1,1,100,1);
		strengthspinner = new JSpinner(strengthmod);
		strengthspinner.setEditor(new JSpinner.NumberEditor(strengthspinner, "0"));
		strengthspinner.setToolTipText(languageResource.getString("strengthcriTooltip"));
		strengthfield = ((JSpinner.DefaultEditor) strengthspinner.getEditor()).getTextField();
		strengthfield.setFont(new Font("Dialog",Font.PLAIN,12));
		strengthspinner.setPreferredSize(new Dimension( 80, 22 ));
		c.gridx = 1;
	    c.gridwidth = 3;
		criteria.add(strengthspinner, c);
		c.gridwidth = 1;
	    c.gridx = 0;
	    c.gridy = 2;
	    criteria.add(Box.createVerticalStrut(20),c);
	    c.gridwidth = 1;
	    c.gridx = 0;
	    c.gridy = 3;
	    criteria.add(Box.createVerticalStrut(20),c);
	    
		//	Constraints Panel
		JPanel constraints = new JPanel();
		c = new GridBagConstraints();
		c.insets = new Insets( 2, 2, 2, 2 );
		c.anchor = GridBagConstraints.WEST;
		constraints.setLayout(new GridBagLayout());
		constraints.setBorder(new CompoundBorder(new TitledBorder(
				new EtchedBorder(), languageResource.getString("constraintsTitle")),
				new EmptyBorder(1, 5, 3, 5)));
		
		//Compatibility
		compatibility = new JCheckBox(languageResource.
				getString("constraintCompatibility"), null, true);
		compatibility.setToolTipText(languageResource.
				getString("constraintCompatibilityTooltip"));
		
		constraints.add(compatibility,c);
		
		//Heel
		heel = new JCheckBox(languageResource.getString("constraintHeel"), null, true);
		heel.setToolTipText(languageResource.
				getString("constraintHeelTooltip"));
	    c.gridwidth = 1;
	    c.gridx = 0;
	    c.gridy = 1;
		constraints.add(heel,c);
		
		//Trim
		trim = new JCheckBox(languageResource.getString("constraintTrim"), null, true);
		trim.setToolTipText(languageResource.
				getString("constraintTrimTooltip"));
		trim.addItemListener(new ItemListener(){
			public void itemStateChanged(ItemEvent e) {
				if(trim.isSelected()){
					trimfield.setEnabled(true);
					trimspinner.setEnabled(true);
				} else {
					trimfield.setEnabled(false);
					trimspinner.setEnabled(false);
				}
			}
		});
		c.gridwidth = 1;
	    c.gridx = 0;
	    c.gridy = 2;
		constraints.add(trim,c);
		SpinnerNumberModel trimmod = new SpinnerNumberModel(0,0,0.5,0.01);
		trimspinner = new JSpinner(trimmod);
		trimspinner.setEditor(new JSpinner.NumberEditor(trimspinner, "0.00"));
		trimspinner.setToolTipText(languageResource.getString("trimconTooltip"));
		trimfield = ((JSpinner.DefaultEditor) trimspinner.getEditor()).getTextField();
		trimfield.setFont(new Font("Dialog",Font.PLAIN,12));
		trimspinner.setPreferredSize(new Dimension( 80, 22 ));
		c.gridx = 1;
	    c.gridwidth = 3;
	    constraints.add(trimspinner, c);
		
		//Stability
		stability = new JCheckBox(languageResource.getString("constraintStability")
				, null, true);
		stability.setToolTipText(languageResource.
				getString("constraintStabilityTooltip"));
		stability.addItemListener(new ItemListener(){
			public void itemStateChanged(ItemEvent e) {
				if(stability.isSelected()){
					stabfield.setEnabled(true);
					stabspinner.setEnabled(true);
				} else {
					stabfield.setEnabled(false);
					stabspinner.setEnabled(false);
				}
			}
		});
		c.gridwidth = 1;
	    c.gridx = 0;
	    c.gridy = 3;
		constraints.add(stability,c);
		stabspinner = new JSpinner(new SpinnerNumberModel(0.30,0.01,0.50,0.01));
		stabspinner.setToolTipText(languageResource.getString("stabconTooltip"));
		OptionalPercentSpinEditor editor = new OptionalPercentSpinEditor(stabspinner);
		stabspinner.setEditor(editor);
		stabfield = ((JSpinner.DefaultEditor) stabspinner.getEditor()).getTextField();
		stabfield.setFont(new Font("Dialog",Font.PLAIN,12));
		stabspinner.setPreferredSize(new Dimension( 80, 22 ));
		c.gridx = 1;
	    c.gridwidth = 3;
	    constraints.add(stabspinner, c);
		
	    // Which solver
		JPanel solverpanel = new JPanel();
		solverpanel.setLayout(new BoxLayout(solverpanel, BoxLayout.X_AXIS));
		solverpanel.setBorder(new CompoundBorder(new TitledBorder(
				new EtchedBorder(), languageResource.getString("solverTitle")), 
				new EmptyBorder(1, 5, 3, 5)));

		String[] supportedSolvers = { "GNU GLPK","LP Solve"};
		solverbox = new JComboBox(supportedSolvers);
		solverbox.setToolTipText(languageResource.getString("solverTooltip"));
		solverbox.setAlignmentX(SwingConstants.LEFT);
		solverpanel.add(solverbox);
		
		JPanel main = new JPanel();
		main.setLayout(new BorderLayout());
		main.add(BorderLayout.WEST, criteria);
		main.add(BorderLayout.CENTER,constraints);
		main.add(BorderLayout.SOUTH,solverpanel);
		
		setLayout(new BorderLayout());
		add(BorderLayout.NORTH,main);
	}
	
	//	 ItemListener that checks that at least one criteria is selected
	class CriteriaListener implements ItemListener {
		public void itemStateChanged(ItemEvent e) {
			JCheckBox source = (JCheckBox) e.getSource();
			if ((strength.isSelected() == false)
					&& (usedtanks.isSelected() == false)) {
				source.setSelected(true);
				JOptionPane.showMessageDialog(Settings.getInstance().getDialog(),
						languageResource.getString("messageCriteria"));
			}
		}
	}
	
	private static class OptionalPercentFormat extends DecimalFormat {
		public OptionalPercentFormat() {
			super("0%");
			setMultiplier(100);
		}

		public Object parseObject(String source) throws ParseException {
			if (!source.endsWith("%"))
				source += "%";
			return super.parseObject(source);
		}
	}

	private static class OptionalPercentSpinEditor extends
			JSpinner.DefaultEditor {
		public OptionalPercentSpinEditor(JSpinner spinner) {
			super(spinner);
			NumberFormatter formatter = new NumberFormatter(
					new OptionalPercentFormat());
			formatter.setValueClass(Double.class);
			JFormattedTextField ftf = getTextField();
			ftf.setFormatterFactory(new DefaultFormatterFactory(formatter));
			ftf.setEditable(true);
			ftf.setHorizontalAlignment(JTextField.RIGHT);
		}
	}
}
