//: /be/optiloading/ship/StrengthGraph.java
/*
 * This work is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2007 Tim Pintens. All rights reserved.
 */

package be.optiloading.ship;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.geom.GeneralPath;

import javax.swing.JPanel;

/**
 * StrengthGraph to show strength curves.
 * 
 * @author Tim Pintens, info@optiloading.be
 * @author <a target="_blank"
 *         href="http://www.optiloading.be">www.optiloading.be</a>
 */

@SuppressWarnings("serial")
public class StrengthGraph extends JPanel{
	private ShipData data;
	private ShipParticular sforbm;
	private ShipParticular max;
	private ShipParticular min;

	/**
	 * Constructor.
	 */
	public StrengthGraph(ShipParticular sforbm, ShipParticular max, ShipParticular min){
		data = ShipData.getInstance();
		this.sforbm = sforbm;
		this.max = max;
		this.min = min;
	}
	
	/**
	 * Paint the Strength force curve.
	 */
	public void paintComponent(Graphics g){
		//Graph settings
		int width = this.getSize().width;
		int height =  this.getSize().height;
		int distfromside = 15;
		int leftmarge = 40;
		int arrowdist = 15;
		
		float xdiv = (width-distfromside-leftmarge-arrowdist)/data.getLoa();
		int xzero = Math.round(leftmarge + Math.abs(data.getBegindist())*xdiv);
		float yrange = Math.abs(data.getShipParticular(max)) + 
				Math.abs(data.getShipParticular(min));
		float ydiv = (height-2*distfromside-arrowdist)/yrange;
		int yzero = Math.round(distfromside + arrowdist + 
				Math.abs(data.getShipParticular(max))*ydiv);
		
		Graphics2D g2d = (Graphics2D)g;
		RenderingHints rh = new RenderingHints(
		RenderingHints.KEY_INTERPOLATION,
		RenderingHints.VALUE_INTERPOLATION_BICUBIC);
		rh.put(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		rh.put(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY);
		g2d.setRenderingHints(rh);
		
		g2d.setColor(new Color(194,217,240));
		g2d.fillRect(2, 2, width-4, height-4); //Background
		
		g2d.setColor(new Color(12,72,133));
		g2d.setStroke(new BasicStroke(1.5f, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND));
		
		//X-axis
		g2d.drawLine(leftmarge, yzero, width-distfromside, yzero);
		g2d.drawLine(width-distfromside-5, yzero-5,width-distfromside, yzero);
		g2d.drawLine(width-distfromside, yzero, width-distfromside-5, yzero+5);
		
		//Y-Axis
		g2d.drawLine(leftmarge, distfromside, leftmarge, height-distfromside);
		g2d.drawLine(leftmarge-5, distfromside+5, leftmarge, distfromside);
		g2d.drawLine(leftmarge, distfromside, leftmarge+5, distfromside+5);

		
		//X-axis markings
		g2d.setFont(new Font("Dialog",Font.PLAIN,10));
		float xstepf = data.getLoa()/100;
		int xstep = Math.round(xstepf) * 10;
		g2d.drawLine(xzero, yzero-5,xzero, yzero+5);
		g2d.drawString("0", xzero-3, yzero+15);
		for(int i = xstep;i <= new Float(data.getEnddist()).intValue();i = i + xstep){
			g2d.drawLine(Math.round(xzero+i*xdiv), yzero-5, 
					Math.round(xzero+i*xdiv), yzero+5);
			g2d.drawString(String.valueOf(i), (Math.round(xzero+i*xdiv)-5), 
					yzero+15);
		}

		//Y-axis markings
		g2d.drawLine(leftmarge-5, yzero,leftmarge, yzero);
		g2d.drawString("0", leftmarge-25, yzero+3);
		
		float ystepf = yrange/10f;
		int len = String.valueOf(Math.round(ystepf)).length();
		int pow = len -1;
		float ystepf2 = (float) (ystepf/Math.pow(10, pow));
		int ystep = (int) (Math.round(ystepf2)*Math.pow(10, pow));
		// Positive markings
		for(int i= 1;(i*ystep) <=data.getShipParticular(max); i++){
			int yvalue = (int) Math.round(yzero-(i*ystep*ydiv));
			g2d.drawLine(leftmarge-5, yvalue, leftmarge+5, yvalue);
			g2d.drawString(String.valueOf(Math.round((i*ystep)/Math.pow(10,pow))), leftmarge-25,
				yvalue+3);
		}
		// Negative markings
		for(int i=1;(i*ystep) <=Math.abs(data.getShipParticular(min)); i++){
			int yvalue = (int) Math.round(yzero+(i*ystep*ydiv));
			g2d.drawLine(leftmarge-5, yvalue, leftmarge+5, yvalue);
			g2d.drawString("-"+String.valueOf(Math.round((i*ystep)/Math.pow(10,pow))), leftmarge-32,
				yvalue+3);
		}

		//Graph
		g2d.setColor(Color.BLUE);
		g2d.setStroke(new BasicStroke(3f, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND));
		GeneralPath path = new GeneralPath();
		path.moveTo(leftmarge,yzero);
		for(int i = 1; i < (width-distfromside-leftmarge-arrowdist+1);i++){
			float dist = (float) ((i* Math.pow(xdiv, -1))+ data.getBegindist());
			int yvalue = Math.round(data.getStrength(sforbm, dist)*ydiv);
			path.lineTo(i+leftmarge, yzero-yvalue);
		}
		g2d.draw(path);
	}

	/**
	 * Get the order of the values. For instance 10^2.
	 * @return Integer with order.
	 */
	public int getPower() {
		float yrange = Math.abs(data.getShipParticular(max)) + 
		Math.abs(data.getShipParticular(min));
		float ystepf = yrange/10f;
		int len = String.valueOf(Math.round(ystepf)).length();
		return (len - 1);
	}

}