package be.optiloading.ship;

import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Component;
import java.awt.Font;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.text.NumberFormat;
import java.util.ResourceBundle;

import javax.swing.BoxLayout;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;

import be.optiloading.settings.Settings;

/**
 * Print a Strength Summary.
 * 
 * @author Tim Pintens, info@optiloading.be
 * @author <a target="_blank"
 *         href="http://www.optiloading.be">www.optiloading.be</a>
 */

@SuppressWarnings("serial")
public class StrengthSummary extends JPanel{
	private static String SFGRAPH;
	private static String BMGRAPH;
	private JPanel graphPanel;
	private JComboBox choosesforbm;
	private StrengthGraph bm;
	private StrengthGraph sf;
	private ResourceBundle languageResource;
	private JLabel yaxis;
	private JLabel maxbm;
	private JLabel maxsf;
	private JLabel maxbmdist;
	private JLabel maxsfdist;
	private ShipData data;
	
	/**
	 * Constructor to build panel
	 */
	public StrengthSummary(){
		data = ShipData.getInstance();
		languageResource = Settings.getInstance().getResourceBundle();
		SFGRAPH = languageResource.getString("sfgraph");
		BMGRAPH = languageResource.getString("bmgraph");
		
		setLayout(new BorderLayout());
		
		graphPanel = new JPanel(new CardLayout());
		graphPanel.setBorder(new EtchedBorder());
		sf = new StrengthGraph(ShipParticular.SF, ShipParticular.MAX_SF,
				ShipParticular.MIN_SF);
		bm = new StrengthGraph(ShipParticular.BM, ShipParticular.MAX_BM,
				ShipParticular.MIN_BM);
		graphPanel.add(sf, SFGRAPH);
		graphPanel.add(bm, BMGRAPH);
		
		JPanel graphInfo = new JPanel();
		graphInfo.setLayout(new BorderLayout());
		String[] options = {SFGRAPH, BMGRAPH};
		
		JPanel choose = new JPanel();
		choose.setLayout(new BoxLayout(choose,BoxLayout.Y_AXIS));
		choosesforbm = new JComboBox(options);
		choosesforbm.setToolTipText(languageResource.getString("chooseTooltip"));
		choosesforbm.setAlignmentY(Component.TOP_ALIGNMENT);
		choosesforbm.addItemListener(new comboItemListener());
		JLabel approximation = new JLabel(languageResource.getString("approximation"));
		approximation.setFont(new Font("Dialog",Font.PLAIN,9));
		choose.add(choosesforbm);
		choose.add(approximation);
		
		JPanel legenda = new JPanel();
		legenda.setLayout(new BoxLayout(legenda,BoxLayout.Y_AXIS));
		legenda.setBorder(new CompoundBorder(new TitledBorder(new EtchedBorder(),
		languageResource.getString("legenda")), new EmptyBorder(5, 5, 5, 5)));
		JLabel xaxis = new JLabel(languageResource.getString("xaxis"));
		yaxis = new JLabel();
		legenda.add(xaxis);
		legenda.add(yaxis);
		
		JPanel maxmin = new JPanel();
		maxmin.setLayout(new BoxLayout(maxmin, BoxLayout.Y_AXIS));
		maxmin.setBorder(new CompoundBorder(new TitledBorder(new EtchedBorder(),
				languageResource.getString("maxmin")), new EmptyBorder(5, 5, 5, 5)));
		maxsf = new JLabel();
		maxsfdist = new JLabel();
		maxbm = new JLabel();
		maxbmdist = new JLabel();
		maxmin.add(maxsf);
		maxmin.add(maxsfdist);
		maxmin.add(maxbm);
		maxmin.add(maxbmdist);
		updateLabels();
		
		graphInfo.add(BorderLayout.NORTH, choose);
		graphInfo.add(BorderLayout.CENTER,legenda);
		graphInfo.add(BorderLayout.SOUTH, maxmin);
		
		JPanel graphInfo2 = new JPanel(new BorderLayout());
		graphInfo2.add(BorderLayout.NORTH,graphInfo);
		
		add(BorderLayout.CENTER,graphPanel);
		add(BorderLayout.EAST, graphInfo2);
		this.addComponentListener(new ComponentAdapter(){
			public void componentResized(ComponentEvent e) {
				updateLabels();
			}
		});
	}
	
	/**
	 * Update labels
	 */
	private void updateLabels(){
		String yaxissf = languageResource.getString("yaxis")+ " SF in 10<sup>"+ 
		String.valueOf(sf.getPower())+"</sup> t</html>";
		String yaxisbm = languageResource.getString("yaxis")+ " BM in 10<sup>"+ 
		String.valueOf(bm.getPower())+"</sup> t.m</html>";
		
		if(choosesforbm.getSelectedIndex() == 0){
			yaxis.setText(yaxissf);
		} else {
			yaxis.setText(yaxisbm);
		}
		NumberFormat formatter = NumberFormat.getInstance();
		formatter.setMaximumFractionDigits(2);
		
		String smaxsf = languageResource.getString("maxsf") + " " + formatter.format(data.getAbsMaxSF());
		String smaxsfdist = languageResource.getString("maxsfdist") + " " + formatter.format(data.getAbsMaxSFLength());
		String smaxbm = languageResource.getString("maxbm") + " " + formatter.format(data.getAbsMaxBM());
		String smaxbmdist = languageResource.getString("maxbmdist") + " " + formatter.format(data.getAbsMaxBMLength());
		
		maxsf.setText(smaxsf);
		maxsfdist.setText(smaxsfdist);
		maxbm.setText(smaxbm);
		maxbmdist.setText(smaxbmdist);
	}
	
	// Show SF or BM graph
	private class comboItemListener implements ItemListener{
		public void itemStateChanged(ItemEvent e) {
			CardLayout cl = (CardLayout)(graphPanel.getLayout());
		    cl.show(graphPanel, (String)e.getItem());
		    updateLabels();
		}
	}
	
	/**
	 * Update panel
	 */
	public void update(){
		bm.repaint();
		sf.repaint();
		updateLabels();
	}
}
