//: /be/optiloading/tank/TankRecord.java
/*
 * This work is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2007 Tim Pintens. All rights reserved.
 */

package be.optiloading.tank;

/**
 * TankRecord class used to stow information about the tank.
 * 
 * @author Tim Pintens, info@optiloading.be
 * @author <a target="_blank"
 *         href="http://www.optiloading.be">www.optiloading.be</a>
 */

public class TankRecord {
	private float ullage;
	private float volume;
	private float lcg;
	private float vcg;
	private float tcg;
	private float fsm;
	
	/**
	 * Constructor of tank record
	 * @param ullage Ullage value.
	 * @param volume Volume.
	 * @param lcg LCG value.
	 * @param vcg VCG value.
	 * @param tcg TCG value.
	 * @param fsm FSM value.
	 */
	public TankRecord(float ullage, float volume, float lcg, 
			float vcg, float tcg, float fsm){
		this.ullage = ullage;
		this.volume = volume;
		this.lcg = lcg;
		this.vcg = vcg;
		this.tcg = tcg;
		this.fsm = fsm;
	}
	
	/**
	 * Method to get the tank's ullage
	 * 
	 * @return Float value containing the tank's ullage
	 */
	public float getUllage() {
		return ullage;
	}
	
	/**
	 * Method to get the volume of the cargo in the tank
	 * 
	 * @return Floating point value containing the volume of the cargo in the
	 *         tank
	 */
	public float getVolume() {
		return volume;
	}
	
	/**
	 * Method to get the Longitudinal Center of Gravity (LCG) of the tank
	 * 
	 * @return Floating point value containing the LCG of the tank
	 */
	public float getLCG() {
		return lcg;
	}

	/**
	 * Method to get the VCG of the tank
	 * 
	 * @return Floating point value containing the KG of the tank
	 */
	public float getVCG() {
		return vcg;
	}
	
	/**
	 * Method to get the VCG of the tank
	 * 
	 * @return Floating point value containing the KG of the tank
	 */
	public float getTCG() {
		return tcg;
	}

	/**
	 * Method to get the Longitudinal Free Surface Moment (FSM) of the tank
	 * 
	 * @return Floating point value containing the FSM of the tank
	 */
	public float getFSM() {
		return fsm;
	}
	
}
